function x_integrated = forward_euler(odefun,x0,u,d,noise,Ts)


% Forward euler implementation. Fixed time and assumed time-invariant
% ordinary differential equation
%
% x_integrated = forward_euler(@odefun,x0,u,d,noise,Ts)
%
% INPUTS:
% @odefun - function handle of the derivative function x' = odefun(x,u)
% x0 - lower boundary of integration
% u - input, assumed constant during timestep
% noise - full-state noise variables, if zero set to 0
% Ts - integration time
%
% OUTPUT:
% x_integrated - value after integration

n_s = size(d,2);

x_dot = odefun(x0,u,d) + noise;

if n_s > 1
    x_integrated = repmat(x0,1,n_s) + Ts*x_dot;
else
    x_integrated = x0 + Ts*x_dot;
end