function x_integrated = runge_kutta_4(odefun,x0,u,d,noise,Ts)


% Runge Kutta 4 implementation. Fixed time and assumed time-invariant
% ordinary differential equation
%
% x_integrated = runge_kutta_4(@odefun,x0,u,d,noise,Ts)
%
% INPUTS:
% @odefun - function handle of the derivative function x' = odefun(x,u)
% x0 - lower boundary of integration
% u - input, assumed constant during timestep
% noise - full-state noise variables, if zero set to 0
% Ts - integration time
%
% OUTPUT:
% x_integrated - value after integration

k1 = feval(odefun,x0,u,d);
k2 = feval(odefun,x0 + Ts*k1/2,u,d);
k3 = feval(odefun,x0 + Ts*k2/2,u,d);
k4 = feval(odefun,x0 + Ts*k3,u,d);

x_dot = (k1 + 2*k2 + 2*k3 + k4)/6 + noise;


%% Add noise

x_integrated = x0 + Ts*x_dot;