function [u,result] = NMPC(u_0,Prob,result)

if strcmp(Prob.controller.CONSTRAINED,'On')
    A = Prob.controller.A;
    B = Prob.controller.B;
    Aeq = Prob.controller.Aeq;
    Beq = Prob.controller.Beq;
    u_L = Prob.controller.u_L;
    u_U = Prob.controller.u_U;
    OPTIONS = Prob.controller.options;
%     u = fmincon(@objective_function,u_0,A,B,Aeq,Beq,u_L,u_U,@state_constraint,OPTIONS);
u = fmincon(@objective_function,u_0,A,B,Aeq,Beq,u_L,u_U,[],OPTIONS);
else
    OPTIONS = Prob.controller.options;
    u = fminunc(@objective_function,u_0,OPTIONS);
end


    function f = objective_function(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0; % initialverdi
        n_x = Prob.simulation.n_x; % dim states
        n_u = Prob.simulation.n_u; % dim input

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = Prob.ref.ref_dimension;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q; % State deviation weighting
        R = Prob.controller.R; % Input use weighting
        S = Prob.controller.S; % Input change weighting

        % Desired path
        z_ref = zeros(n_p*n_r,1);
        for l = 1:n_p
            z_ref((l-1)*n_r + 1:l*n_r) = result.ref(:,l,k);
        end

        % Input blocking
        u = diag([x(end-2) x(end-1) x(end)])*ones(3,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)*n_u + 1:l*n_u);
            end
        end


        % BIAS
        %         if k > 1
        %             ref_bias = z_0(1:n_r) - result.z_predicted(1:n_r,1,k-1);
        %         else
        %             ref_bias = zeros(n_r,1);
        %         end

        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(n_x,n_p);
        z_p = zeros(n_p*n_r,1);

        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,1),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l);% + ref_bias; % Measurement function + bias
        end


        % Return f, cost
%         x_change = zeros((n_p-1)*n_u,1);
%         if k > 1;  x_change(1:3) = x(1:3) - result.u(:,k-1);
%         else x_change(1:3) = x(1:3); end
%         for l = 2:n_p-1
%             x_change((l-1)*n_u + 1:l*n_u ) = x((l-1)*n_u+1:l*n_u) - x((l-2)*n_u+1:(l-1)*n_u);
%         end



        % Save information
         result.f(k) = (z_p-z_ref)'*Q*(z_p-z_ref); % + x'*R*x + x_change'*S*x_change;
        result.z_predicted(:,:,k) = z_predicted;
        f = result.f(k);
    end
function [c,ceq] = state_constraint(x)
        z_0 = Prob.controller.x_0; % initialverdi
        n_x = Prob.simulation.n_x; % dim states
        n_u = Prob.simulation.n_u; % dim input

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = 3;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep

        % Input blocking
        u = diag([x(end-2) x(end-1) x(end)])*ones(3,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)*n_u + 1:l*n_u);
            end
        end

        z_predicted = zeros(n_x,n_p);
        z_p = zeros(n_p*n_r,1);

        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,1),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l);% + ref_bias; % Measurement function + bias
        end

        % Bounds
        % Desired path
        z_ref = zeros(n_p*n_r,1);
        for l = 1:n_p
            z_ref((l-1)*n_r + 1:l*n_r) = result.ref(:,l,k);
        end
        
        position_line = [Prob.controller.x_con; Prob.controller.y_con; Prob.controller.psi_con];
        k_max = repmat(position_line,n_p,1);
        k_min = -k_max;

        cineq_pos = z_p - z_ref - k_max;
        cineq_neg = k_min - z_p + z_ref;
        
        

        ceq = []; %
        c = [cineq_pos; cineq_neg];

    end
end