function [u,result] = NMPC(u_0,Prob,result)

if strcmp(Prob.controller.CONSTRAINED,'On')
    A = Prob.controller.A;
    B = Prob.controller.B;
    Aeq = Prob.controller.Aeq;
    Beq = Prob.controller.Beq;
    u_L = Prob.controller.u_L;
    u_U = Prob.controller.u_U;
    NONLCON = Prob.controller.NONLCON;
    OPTIONS = Prob.controller.options;
    
    if strcmp(Prob.controller.decoupled,'On')

    u_L1 = u_L(1:3:end);
    u_U1 = u_U(1:3:end);
    u_01 = u_0(1:3:end);
    u_L2 = repmat(u_L(2:3),Prob.controller.n_p,1);
    u_U2 = repmat(u_U(2:3),Prob.controller.n_p,1);
    u_02 = repmat(u_0(2:3),Prob.controller.n_p,1);
    
    Prob.controller.Q1 = blkdiag2(Prob.controller.Q(1),Prob.controller.n_p);
    Prob.controller.R1 = blkdiag2(Prob.controller.R(1),Prob.controller.n_p);
    Prob.controller.S1 = blkdiag2(Prob.controller.S(1),Prob.controller.n_p);    
    Prob.controller.Q2 = blkdiag2(Prob.controller.Q(2:3,2:3),Prob.controller.n_p);
    Prob.controller.R2 = blkdiag2(Prob.controller.R(2:3,2:3),Prob.controller.n_p);
    Prob.controller.S2 = blkdiag2(Prob.controller.S(2:3,2:3),Prob.controller.n_p);    
    u(1,:) = fmincon(@objective_function_1,u_01,A,B,Aeq,Beq,u_L1,u_U1,NONLCON,OPTIONS);
    u(2:3,:) = zeros(2,Prob.controller.n_p);
%     u(2:3,:) = fmincon(@objective_function_2,u_02(1:3:end),A,B,Aeq,Beq,u_L2,u_U2,NONLCON,OPTIONS);
    else
    u = fmincon(@objective_function_3,u_0,A,B,Aeq,Beq,u_L,u_U,NONLCON,OPTIONS);
    end
else
    OPTIONS = Prob.controller.options;
    u = fminunc(@objective_function,u_0,OPTIONS);
end
function f = objective_function_1(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0(1); % initialverdi
        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0(1); % current disturbance
        b_0 = Prob.controller.b_0(1); % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q1; % State deviation weighting
        R = Prob.controller.R1; % Input use weighting
        S = Prob.controller.S1; % Input change weighting

        % Desired path
        z_ref = zeros(n_p,1);  
        for l = 1:n_p
            z_ref((l-1) + 1:l) = result.ref(1,l,k);
        end

        % Input blocking
        u = diag(x(end))*ones(1,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)+ 1:l);
            end
        end
       
        % BIAS
        if k > 1  ref_bias = z_0(1) - result.z_predicted(1,1,k-1);
        else ref_bias = 0;  end
        
        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(1,n_p);  z_p = zeros(n_p,1);

        for l = 1:n_p
            if l == 1
                z_predicted(1) = feval(Prob.controller.integrator,@ffun_surge,z_0,u(:,1),[d_0; b_0],0,Ts);
            else
                z_predicted(l) = feval(Prob.controller.integrator,@ffun_surge,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l) + ref_bias; % Measurement function + bias
        end
        
        
        % Return f, cost
        x_change = zeros((n_p-1),1);
        if k > 1;  x_change(1) = x(1) - result.u(1,k-1);
        else x_change = x(1); end
        for l = 2:n_p-1  x_change((l-1) +  1: l ) = x((l-1)+1:l) - x((l-2)+1:(l-1));  end
        
        % Save information
        result.f(k) = (z_p-z_ref)'*Q*(z_p-z_ref) + x'*R*x + x_change'*S*x_change;
        result.z_predicted(1,:,k) = z_predicted;
        f = result.f(k);
end

function f = objective_function_2(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0(2:3); % initialverdi

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = 2;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q2; % State deviation weighting
        R = Prob.controller.R2; % Input use weighting
        S = Prob.controller.S2; % Input change weighting

        % Desired path
        z_ref = zeros(n_p*n_r,1); 
        for l = 1:n_p
            z_ref((l-1)*n_r + 1:l*n_r) = result.ref(2:3,l,k);
        end

        % Input blocking
        u = diag([x(end-1) x(end)])*ones(2,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)*2 + 1:l*2);
            end
        end

        
        % BIAS
        if k > 1
            ref_bias = z_0(2:3) - result.z_predicted(2:3,1,k-1);
        else
            ref_bias = zeros(2,1);
        end
        
        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(2,n_p);
        z_p = zeros(n_p*2,1);

        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,1),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(2*(l-1)+1:2*l) = z_predicted(1:2,l) + ref_bias; % Measurement function + bias
        end
        
        
        % Return f, cost
        x_change = zeros((n_p-1)*n_u,1);
        if k > 1;  x_change(1:2) = x(1:2) - result.u(2:3,k-1);
        else x_change(1:3) = x(1:3); end
        for l = 2:n_p-1
            x_change((l-1)*0 +  1: l*0 ) = x((l-1)*0+1:l*0) - x((l-2)*0+1:(l-1)*0);
        end
        
        

        % Save information
        result.f2(k) = (z_p-z_ref)'*Q*(z_p-z_ref) + x'*R*x + x_change'*S*x_change;
        result.z_predicted(2:3,:,k) = z_predicted;
        f = result.f(k);
    end

    function f = objective_function_3(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0; % initialverdi
        n_x = Prob.simulation.n_x; % dim states
        n_u = Prob.simulation.n_u; % dim input

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = Prob.ref.ref_dimension;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q; % State deviation weighting
        R = Prob.controller.R; % Input use weighting
        S = Prob.controller.S; % Input change weighting

        % Desired path
        z_ref = zeros(n_p*n_r,1); 
        for l = 1:n_p
            z_ref((l-1)*n_r + 1:l*n_r) = result.ref(:,l,k);
        end

        % Input blocking
        u = diag([x(end-2) x(end-1) x(end)])*ones(3,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)*n_u + 1:l*n_u);
            end
        end

        
        % BIAS
        if k > 1
            ref_bias = z_0(1:n_r) - result.z_predicted(1:n_r,1,k-1);
        else
            ref_bias = zeros(n_r,1);
        end
        
        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(n_x,n_p);
        z_p = zeros(n_p*n_r,1);

        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,1),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l) + ref_bias; % Measurement function + bias
        end
        
        
        % Return f, cost
        x_change = zeros((n_p-1)*n_u,1);
        if k > 1;  x_change(1:3) = x(1:3) - result.u(:,k-1);
        else x_change(1:3) = x(1:3); end
        for l = 2:n_p-1
            x_change((l-1)*n_u +  1: l*n_u ) = x((l-1)*n_u+1:l*n_u) - x((l-2)*n_u+1:(l-1)*n_u);
        end
        
        

        % Save information
        result.f(k) = (z_p-z_ref)'*Q*(z_p-z_ref) + x'*R*x + x_change'*S*x_change;
        result.z_predicted(:,:,k) = z_predicted;
        f = result.f(k);
    end
end