function [u,Prob] = NMPC(u_0,Prob)

if strcmp(Prob.controller.CONSTRAINED,'On')
    A = Prob.controller.A;
    B = Prob.controller.B;
    Aeq = Prob.controller.Aeq;
    Beq = Prob.controller.Beq;
    u_L = Prob.controller.u_L;
    u_U = Prob.controller.u_U;
    NONLCON = Prob.controller.NONLCON;
    OPTIONS = Prob.controller.options;
    u = fmincon(@objective_function,u_0,A,B,Aeq,Beq,u_L,u_U,NONLCON,OPTIONS);
else
    OPTIONS = Prob.controller.options;
    u = fminunc(@objective_function,u_0,OPTIONS);
end
Prob.controller.u(:,Prob.simulation.k) = u(1:n_u);

    function [f,J,H] = objective_function(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0; % initialverdi
        n_x = Prob.simulation.n_x; % dim states
        n_u = Prob.simulation.n_u; % dim input

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = Prob.ref.ref_dimension;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q; % State deviation weighting
        R = Prob.controller.R; % Input use weighting
        S = Prob.controller.S; % Input change weighting

        % Desired path
        z_ref = zeros(n_p*n_r,1);
        for l = 1:n_p
            z_ref((l-1)*n_r + 1:l*n_r) = Prob.result.ref(:,l,k);
        end

        % Input blocking
        u = diag([x(end-2) x(end-1) x(end)])*ones(3,n_p);
        for l = 1:n_o
            for m = 1:n_b
                u(:,(l-1)*n_b + m) = x((l-1)*n_u + 1:l*n_u);
            end
        end


        % BIAS
        ref_bias = z_0(1:n_r) - Prob.controller.z_prev_predicted(1:n_r);


        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(n_x,n_p);
        z_p = zeros(n_p*n_r,1);

        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,l),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l) + ref_bias; % Measurement function + bias
        end


        % Return f, cost
        x_change = zeros((n_p-1)*n_u,1);
        if k > 1;  x_change(1:3) = x(1:3) - Prob.controller.u(:,k-1);
        else x_change(1:3) = x(1:3); end
        for l = 2:n_p-1
            x_change((l-1)*n_u +  1: l*n_u ) = x((l-1)*n_u+1:l*n_u) - x((l-2)*n_u+1:(l-1)*n_u);
        end



        % Save information
        Prob.result.f(k) = (z_p-z_ref)'*Q*(z_p-z_ref) + x'*R*x + x_change'*S*x_change;
        Prob.result.z_predicted(:,:,k) = z_predicted;
        f = Prob.result.f(k);

        if nargout > 1
            % calculate g
            I = eye(n_x);
            B = [zeros(3); eye(3)];
            A = zeros(n_x,n_x);
            AA = zeros(n_x,n_x,n_p-1);

            for k = 1:n_p
                if k == 1
                    A(:,:) = jacobi_modulus(z_0,d_0);
                else
                    A(:,:) = jacobi_modulus(z_predicted(:,k-1),[d_0; b_0]);
                end
                AA(:,:,k) = (I + Ts*A(:,:));
            end

            v = zeros(n_p*n_x,n_p*n_u);
            for k = 1:n_p
                for l = 1:n_p
                    if k > l
                        aaa = 1;
                        for m = k:-1:l
                            aaa = aaa*AA(:,:,m);
                        end
                    elseif k == l
                        aaa = 1;
                    else
                        aaa = 0;
                    end
                    v( (k-1)*n_x + 1:k*n_x,(l-1)*n_u + 1:l*n_u ) = aaa*Ts*B;
                end
            end

            J = 2*(z_p-z_ref)'*Q*v + 2*x'*R;
        end
        
        if nargout > 2
            H = 2*v'*Q*v + 2*R;
        end

    end
end