function u = NMPC_input_change(u_0,Prob)


if strcmp(Prob.controller.CONSTRAINED,'On')
    A = Prob.controller.A;
    B = Prob.controller.B;
    Aeq = Prob.controller.Aeq;
    Beq = Prob.controller.Beq;
    u_L = Prob.controller.u_L;
    u_U = Prob.controller.u_U;
    NONLCON = Prob.controller.NONLCON;
    OPTIONS = Prob.controller.options;
    u = fmincon(@objective_function,u_0,A,B,Aeq,Beq,u_L,u_U,NONLCON,OPTIONS);
else
    OPTIONS = Prob.controller.options;
    u = fminunc(@objective_function,u_0,OPTIONS);
end

    function f = objective_function(x)
        % Maneuvering problem, free velocity
        z_0 = Prob.controller.x_0; % initialverdi
        n_x = Prob.simulation.n_x; % dim states
        n_u = Prob.simulation.n_u; % dim input

        n_p = Prob.controller.n_p; % length prediction horizon
        n_o = Prob.controller.n_o; % number of optimization variables/n_u how many unique input horizons
        n_b = Prob.controller.n_b; % blocking
        n_r = Prob.ref.ref_dimension;

        % INPUT sizes
        % n_c - when l >= n_c u = fixed
        % n_b*n_o <= n_c
        k = Prob.simulation.k;
        d_0 = Prob.controller.d_0; % current disturbance
        b_0 = Prob.controller.b_0; % bias disturbance
        Ts = Prob.simulation.Ts; % Timestep
        Q = Prob.controller.Q; % State deviation weighting
        R = Prob.controller.R; % Input use weighting
        S = Prob.controller.S; % Input change weighting

        % Desired path
        z_ref = zeros(n_p*n_r,1); 
        for l = 1:n_p; z_ref((l-1)*n_r + 1:l*n_r) = Prob.result.ref(:,l,k); end




        % Input changes
        deltau = zeros(3,n_p); % n_o*n_b <= n_c
        for l = 1:n_o
            for m = 1:n_b
                if m == 1
                    deltau(:,(l-1)*n_b + m) = x_u((l-1)*n_u + 1:l*n_u);
                else
                    deltau(:,(l-1)*n_b + m) = zeros(3,1);
                end
            end
        end
        
        % Total input
        u = zeros(n_u,n_p);
        u(:,1) = Prob.controller.u(:,k-1) + deltau(:,1);
        for l = 2:n_p
            u(:,l) = u(:,l-1) + deltau(:,l);
        end
                    
        % BIAS 
        b = z_0(1:n_r) - Prob.controller.z_prev_predicted(1:n_r);
        
        
        % Integrere opp horisonten, kun tilstandar berekna, pdrag, parametrar og
        % forstyrringar kjende.
        z_predicted = zeros(n_x,n_p); z_p = zeros(n_p*n_r,1); % same vector, matrix and vector form
        
        for l = 1:n_p
            if l == 1
                z_predicted(:,1) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_0,u(:,l),[d_0; b_0],0,Ts);
            else
                z_predicted(:,l) = feval(Prob.controller.integrator,Prob.controller.system_update_function,z_predicted(:,l-1),u(:,l),[d_0; b_0],0,Ts);
            end
            z_p(n_r*(l-1)+1:n_r*l) = z_predicted(1:n_r,l) + b; % Measurement function + bias
        end
        Prob.controller.z_prev_predicted = z_predicted(:,1);
        
        
        deltau_cost = zeros(n_p*n_u,1);
        for l = 1:n_p
            deltau_cost((l-1)*n_u + 1:l*n_u) = deltau(:,l);
        end
        
        % Return f, cost        
        f = (z_p-z_ref)'*Q*(z_p-z_ref) + x'*R*x + deltau_cost'*S*deltau_cost;

        % Save information
        Prob.result.f(k) = f;
        Prob.result.z_predicted(:,:,k) = z_predicted;
    end
    end
end