function [j_states,j_disturbance] = jacobi_abko(x,d)
global cs
A = cs.m - cs.X_udot;
C = cs.m - cs.Y_vdot;
D = cs.m*cs.x_G - cs.Y_rdot;
F = cs.m*cs.x_G - cs.N_vdot;
G = cs.I_z - cs.N_rdot;
L = C*G - D*F;


psi = x(3);
u = x(4);
v = x(5);
r = x(6);
x_cdot = d(1);
y_cdot = d(2);
b_x = d(3);
b_y = d(4);
b_psi = d(5);

c_psi = cos(psi);
s_psi = sin(psi);
u_c = c_psi*x_cdot + s_psi*y_cdot;
v_c = -s_psi*x_cdot + c_psi*y_cdot;
u_r = u - u_c;
v_r = v - v_c;




%% Current
Du_cDpsi = -s_psi*x_cdot + c_psi*y_cdot;
Dv_cDpsi = -c_psi*x_cdot - s_psi*y_cdot;

Du_cDx_cdot = c_psi;
Du_cDy_cdot = s_psi;
Dv_cDx_cdot = -s_psi;
Dv_cDy_cdot = c_psi;


%% x
Dx_dotDpsi = -u*s_psi - v*c_psi;
Dx_dotDu = c_psi;
Dx_dotDv = -s_psi;

%% y
Dy_dotDpsi = u*c_psi - v*s_psi;
Dy_dotDu = s_psi;
Dy_dotDv = c_psi;

%% psi
Dpsi_dotDr = 1;

%% B
DBDu_c = -cs.X2_u - 3*cs.X2_uuu*(u_r)^2;
DBDv_c = cs.Y_vdot*r;
DBDu = cs.X2_u + 3*cs.X2_uuu*(u_r)^2;
DBDv = (cs.m - cs.Y_vdot)*r;
DBDr = 2*cs.m*cs.x_G*r + v - cs.Y_vdot*(v_r) + 0.5*(cs.N_vdot + cs.Y_rdot);
DBDpsi = DBDu_c*Du_cDpsi + DBDv_c*Dv_cDpsi -s_psi*b_x + c_psi*b_y;

%Current
DBDx_cdot = DBDu_c*Du_cDx_cdot + DBDv_c*Dv_cDx_cdot;
DBDy_cdot = DBDu_c*Du_cDy_cdot + DBDv_c*Dv_cDy_cdot;

% Bias force
DBDb_x = c_psi;
DBDb_y = s_psi;
DBDb_psi = 0;

%% E
DEDu_c = -cs.X_udot*r;
DEDv_c = -cs.Y2_v - 3*cs.Y2_vvv*(v_r)^2 - 2*cs.Y2_vvr*(v_r)*r - cs.Y2_vrr*r^2;
DEDu = -cs.m*r + cs.X_udot*r;
DEDv = cs.Y2_v + 3*cs.Y2_vvv*(v_r)^2 + 2*cs.Y2_vvr*(v_r)*r + cs.Y2_vrr*r^2;
DEDr = -cs.m*u + cs.X_udot*(u_r) + cs.Y2_r + cs.Y2_vvr*(v_r)^2 + 2*cs.Y2_vrr*(v_r)*r + 3*cs.Y2_rrr*r^2;
DEDpsi = DEDu_c*Du_cDpsi + DEDv_c*Dv_cDpsi - c_psi*b_x - s_psi*b_y; 

% Current
DEDx_cdot = DBDu_c*Du_cDx_cdot + DBDv_c*Dv_cDx_cdot;
DEDy_cdot = DBDu_c*Du_cDy_cdot + DBDv_c*Dv_cDy_cdot;

% Bias force
DEDb_x = -s_psi;
DEDb_y = c_psi;
DEDb_psi = 0;



%% H
DHDu_c = cs.X_udot*(v_r) - 0.5*(cs.N_vdot + cs.Y_rdot) - cs.Y_vdot*(v_r);
DHDv_c = -cs.Y_vdot*(u_r) + cs.X_udot*(u_r) -cs.N2_v - 3*cs.N2_vvv*(v_r^2) - 2*cs.N2_vvr*(v_r)*r - cs.N2_vrr*r^2;
DHDu = -cs.m*cs.x_G*r - cs.X_udot*(v_r) + 0.5*(cs.N_vdot + cs.Y_rdot) + cs.Y_vdot*(v_r);
DHDv = -DHDv_c;
DHDr = -cs.m*cs.x_G*u + cs.N2_r + cs.N2_vvr*(v_r)^2 + 2*cs.N2_vrr*(v_r) + 3*cs.N2_rrr*r^2;
DHDpsi = DHDu_c*Du_cDpsi + DHDv_c*Dv_cDpsi;


% Current
DHDx_cdot = DHDu_c*Du_cDx_cdot + DHDv_c*Dv_cDx_cdot;
DHDy_cdot = DHDu_c*Du_cDy_cdot + DHDv_c*Dv_cDy_cdot;

% Bias force
DHDb_x = 0;
DHDb_y = 0;
DHDb_psi = 1;



%% Create matrices
j_states = zeros(6);
j_states(1,3:6) = [Dx_dotDpsi Dx_dotDu Dx_dotDv 0];
j_states(2,3:6) = [Dy_dotDpsi Dy_dotDu Dy_dotDv 0];
j_states(3,3:6) = [0          0        0        Dpsi_dotDr];
j_states(4,3:6) = [DBDpsi DBDu DBDv DBDr]/A;
j_states(5,3:6) = (G*[DEDpsi DEDu DEDv DEDr] - D*[DHDpsi DHDu DHDv DHDr])/L;
j_states(6,3:6) = (C*[DHDpsi DHDu DHDv DHDr] - F*[DEDpsi DEDu DEDv DEDr])/L;
    
if nargout > 1
    j_disturbance = zeros(6,5);
    j_disturbance(4,1:2) = [DBDx_cdot DBDy_cdot]/A;
    j_disturbance(5,1:2) = (G*[DEDx_cdot DEDy_cdot] - D*[DHDx_cdot DHDy_cdot])/L;
    j_disturbance(6,1:2) = (C*[DHDx_cdot DHDy_cdot] - F*[DEDx_cdot DEDy_cdot])/L;
    
    j_disturbance(4,3:5) = [DBDb_x DBDb_y DBDb_psi]/A;
    j_disturbance(5,3:5) = (G*[DEDb_x DEDb_y DEDb_psi] - D*[DEDb_x DEDb_y DEDb_psi])/L;
    j_disturbance(6,3:5) = (C*[DHDb_x DHDb_y DHDb_psi] - F*[DHDb_x DHDb_y DHDb_psi])/L;   
end

if nargout > 2
    j_input = [zeros(3); eye(3)];
end