function [j_states,j_disturbance,j_input] = jacobi_modulus(x,d)
global cs
A = cs.m - cs.X_udot;
C = cs.m - cs.Y_vdot;
D = cs.m*cs.x_G - cs.Y_rdot;
F = cs.m*cs.x_G - cs.N_vdot;
G = cs.I_z - cs.N_rdot;
L = G*C-D*F;

psi = x(3);
u = x(4);
v = x(5);
r = x(6);
x_cdot = d(1);
y_cdot = d(2);

if size(d,1) == 5
    b_x = d(3);
    b_y = d(4);
    b_psi = d(5);
else
    b_x = 0;
    b_y = 0;
    b_psi = 0;
end

c_psi = cos(psi);
s_psi = sin(psi);
u_c = c_psi*x_cdot + s_psi*y_cdot;
v_c = -s_psi*x_cdot + c_psi*y_cdot;
u_r = u - u_c;
v_r = v - v_c;


%% Current
Du_cDpsi = -s_psi*x_cdot + c_psi*y_cdot;
Dv_cDpsi = -c_psi*x_cdot - s_psi*y_cdot;

Du_cDx_cdot = c_psi;
Du_cDy_cdot = s_psi;
Dv_cDx_cdot = -s_psi;
Dv_cDy_cdot = c_psi;

%% x
Dx_dotDpsi = -u*s_psi - v*c_psi;
Dx_dotDu = c_psi;
Dx_dotDv = -s_psi;

%% y
Dy_dotDpsi = u*c_psi - v*s_psi;
Dy_dotDu = s_psi;
Dy_dotDv = c_psi;

%% psi
Dpsi_dotDr = 1;
%% B, surge
%B = (tau(1) - cs.X_udot*u_cdot + cs.m*(cs.x_G*r + v)*r - (cs.Y_vdot*v_r +
%     (cs.N_vdot + cs.Y_rdot)/2*r)*r + (cs.X_u + cs.X_uu*abs(u_r) +
%     cs.X_uuu*u_r^2)*u_r + c_psi*b_x + s_psi*b_y);

DBDu_c = - cs.X_u - 2*cs.X_uu*abs(u_r) - 3*cs.X_uuu*u_r^2;
DBDv_c = cs.Y_vdot*r;

DBDu = -DBDu_c;
DBDv = -DBDv_c + cs.m*r;

DBDpsi = DBDu_c*Du_cDpsi + DBDv_c*Dv_cDpsi -s_psi*b_x + c_psi*b_y;
DBDr = cs.m*v + 2*r*(cs.m*cs.x_G - (cs.N_vdot + cs.Y_rdot)/2) - cs.Y_vdot*v_r;

%Current
DBDx_cdot = DBDu_c*Du_cDx_cdot + DBDv_c*Dv_cDx_cdot;
DBDy_cdot = DBDu_c*Du_cDy_cdot + DBDv_c*Dv_cDy_cdot;

% Bias force
DBDb_x = c_psi;
DBDb_y = s_psi;
DBDb_psi = 0;

%% E, sway yaw
%E = (tau(2) - cs.Y_vdot*v_cdot - cs.m*u*r + cs.X_udot*u_r*r + cs.Y_r*r +
%     cs.Y_v*v_r + (cs.Y_vv*abs(v_r) + cs.Y_rv*abs(r))*v_r + (cs.Y_vr*abs(v_r) +
%     cs.Y_rr*abs(r))*r - s_psi*b_x + c_psi*b_y);
DEDu_c = -cs.X_udot*r;
DEDv_c = cs.Y_v + cs.Y_rv*abs(r) + 2*cs.Y_vv*abs(v_r) + cs.Y_vr*sign(v_r)*r;

DEDu = -cs.m*r - DEDu_c;
DEDv = -DEDv_c;

DEDpsi = DEDu_c*Du_cDpsi + DEDv_c*Dv_cDpsi - c_psi*b_x - s_psi*b_y;
DEDr = -cs.m*u + cs.X_udot*u_r + cs.Y_r + cs.Y_rv*sign(r)*v_r+ cs.Y_vr*abs(v_r) + 2*cs.Y_rr*abs(r);

% Current
DEDx_cdot = DBDu_c*Du_cDx_cdot + DBDv_c*Dv_cDx_cdot;
DEDy_cdot = DBDu_c*Du_cDy_cdot + DBDv_c*Dv_cDy_cdot;

% Bias force
DEDb_x = -s_psi;
DEDb_y = c_psi;
DEDb_psi = 0;

%% H, sway yaw
%H = (tau(3) - cs.N_vdot*v_cdot - cs.m*cs.x_G*r*u + (cs.Y_vdot*v_r +
%     (cs.N_vdot + cs.Y_rdot)/2*r)*u_r - cs.X_udot*u_r*v_r + cs.N_v*v_r +
%     cs.N_r*r + (cs.N_vv*abs(v_r) + cs.N_rv*abs(r))*v_r + (cs.N_vr*abs(v_r) +
%     cs.N_rr*abs(r))*r);
DHDu_c = (cs.X_udot-cs.Y_vdot)*v_r - (cs.N_vdot + cs.Y_rdot)*r/2;
DHDv_c = (cs.X_udot - cs.Y_vdot)*u_r - cs.N_v - 2*cs.N_vv*abs(v_r) - cs.N_vr*sign(v_r)*r;

DHDu = -cs.m*cs.x_G*r - DHDu_c;
DHDv = -DHDv_c;

DHDr = -cs.m*cs.x_G*u + (cs.N_vdot + cs.Y_rdot)/2*u_r + cs.N_r + cs.N_rv*sign(r)*v_r+ cs.N_vr*abs(v_r) + 2*cs.N_rr*abs(r);
DHDpsi = DHDu_c*Du_cDpsi + DHDv_c*Dv_cDpsi; % Skal denne vere med?

% Current
DHDx_cdot = DHDu_c*Du_cDx_cdot + DHDv_c*Dv_cDx_cdot;
DHDy_cdot = DHDu_c*Du_cDy_cdot + DHDv_c*Dv_cDy_cdot;

% Bias force
DHDb_x = 0;
DHDb_y = 0;
DHDb_psi = 1;

%%
j_states = zeros(6);
j_states(1,3:6) = [Dx_dotDpsi Dx_dotDu Dx_dotDv 0];
j_states(2,3:6) = [Dy_dotDpsi Dy_dotDu Dy_dotDv 0];
j_states(3,3:6) = [0          0        0        Dpsi_dotDr];
j_states(4,3:6) = [DBDpsi DBDu DBDv DBDr]/A;
j_states(5,3:6) = (G*[DEDpsi DEDu DEDv DEDr] - D*[DHDpsi DHDu DHDv DHDr])/L;
j_states(6,3:6) = (C*[DHDpsi DHDu DHDv DHDr] - F*[DEDpsi DEDu DEDv DEDr])/L;

if nargout > 1
    if size(d,1) == 5
        j_disturbance = zeros(6,5);
        j_disturbance(4,1:2) = [DBDx_cdot DBDy_cdot]/A;
        j_disturbance(5,1:2) = (G*[DEDx_cdot DEDy_cdot] - D*[DHDx_cdot DHDy_cdot])/L;
        j_disturbance(6,1:2) = (C*[DHDx_cdot DHDy_cdot] - F*[DEDx_cdot DEDy_cdot])/L;

        j_disturbance(4,3:5) = [DBDb_x DBDb_y DBDb_psi]/A;
        j_disturbance(5,3:5) = (G*[DEDb_x DEDb_y DEDb_psi] - D*[DEDb_x DEDb_y DEDb_psi])/L;
        j_disturbance(6,3:5) = (C*[DHDb_x DHDb_y DHDb_psi] - F*[DHDb_x DHDb_y DHDb_psi])/L;
    end

    if size(d,1) == 2
        j_disturbance = zeros(6,2);
        j_disturbance(4,1:2) = [DBDx_cdot DBDy_cdot]/A;
        j_disturbance(5,1:2) = (G*[DEDx_cdot DEDy_cdot] - D*[DHDx_cdot DHDy_cdot])/L;
        j_disturbance(6,1:2) = (C*[DHDx_cdot DHDy_cdot] - F*[DEDx_cdot DEDy_cdot])/L;
    end
end

if nargout > 2
    j_input = [zeros(3); eye(3)];
end
