function z_dot = ffun_sd_ekf(x,tau,d)
% ODE-function implementing Cybership II model. Uses 2. order modulus
% damping. Estimates states and current. 
%
% x_dot = ffun_ukf_sd(x,tau,d)
%
% INPUTS:
% x - complete state of ship. 
%   x(1:3) = eta  
%   x(4:6) = nu
%   x(1:2) = [x_cdot; y_cdot]
% tau - force input
% d = [b_x; b_y; b_psi]
%
% OUTPUT:
% z_dot - the value of differential equation

global cs
A = cs.m - cs.X_udot;
C = cs.m - cs.Y_vdot;
D = cs.m*cs.x_G - cs.Y_rdot;
F = cs.m*cs.x_G - cs.N_vdot;
G = cs.I_z - cs.N_rdot;
L = C*G - D*F;

[n,m] = size(x);
z_dot = zeros(n,m);

mu_x = 0.2;
mu_y = 0.2;

b_x = d(1);
b_y = d(2);
b_psi = d(3);

for k = 1:m

psi = x(3,k);
u = x(4,k);
v = x(5,k);
r = x(6,k);
x_cdot = x(7,k);
y_cdot = x(8,k);

% Rotasjonsmatrise
c_psi = cos(psi);
s_psi = sin(psi);

u_c = c_psi*x_cdot + s_psi*y_cdot;
v_c = -s_psi*x_cdot + c_psi*y_cdot;

u_cdot = r*v_c;
v_cdot = -r*u_c;

% Relative speed
u_r = u - u_c;
v_r = v - v_c;

% Deriverte
u_dot = (tau(1) - cs.X_udot*u_cdot + cs.m*(cs.x_G*r + v)*r - (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)/2*r)*r...
                  + (cs.X_u + cs.X_uu*abs(u_r) + cs.X_uuu*u_r^2)*u_r + c_psi*b_x + s_psi*b_y)/A;  

E = (tau(2) - cs.Y_vdot*v_cdot - cs.m*u*r + cs.X_udot*u_r*r + cs.Y_r*r + cs.Y_v*v_r...
              + (cs.Y_vv*abs(v_r) + cs.Y_rv*abs(r))*v_r + (cs.Y_vr*abs(v_r) + cs.Y_rr*abs(r))*r - s_psi*b_x + c_psi*b_y);
          
H = (tau(3) - cs.N_vdot*v_cdot - cs.m*cs.x_G*r*u + (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)/2*r)*u_r...
              - cs.X_udot*u_r*v_r + cs.N_v*v_r + cs.N_r*r...
              +(cs.N_vv*abs(v_r) + cs.N_rv*abs(r))*v_r + (cs.N_vr*abs(v_r) + cs.N_rr*abs(r))*r + b_psi);
          
v_dot = (G*E-D*H)/L;
r_dot = (C*H-F*E)/L;

x_dot = c_psi*u - s_psi*v;
y_dot = s_psi*u + c_psi*v;
psi_dot = r;


z_dot(:,k) = [x_dot; y_dot; psi_dot; u_dot; v_dot; r_dot; -mu_x*x_cdot; -mu_y*y_cdot];
end
end

  
        
        