function z_dot = ffun_sdw_ekf(x,tau,d)
% ODE-function implementing Cybership II model. Used for estimating states
% with given input and current. Assumes 2. order modulus damping model.
% Current given relative to NED-frame. Current has dynamics!
%
% x_dot = ffun_ukf_s(x,tau,d)
%
% INPUTS:
% x - complete state of ship. 
%   x(1:3) = eta  
%   x(4:6) = nu
% d(1:2) = [V_c; beta_c]
% tau - force input
%
% OUTPUT:
% x_dot - the value of differential equation

global cs

[n_x,n_s] = size(x);
z_dot = zeros(n_x,n_s);
n_d = 2;
n_w = 9;
n_u = 3;
n_y = 3;


w_0x = 0.8;
w_0y = 0.8;
w_0psi = 0.8;
lambda_x = 0.26;
lambda_y = 0.26;
lambda_psi = 0.26;
T_x = 1000;
T_y = 2000;
T_psi = 3000;


Aw = [zeros(3) eye(3);
      diag([-w_0x^2 -w_0y^2 -w_0psi^2]) diag([-2*lambda_x*w_0x -2*lambda_y*w_0y -2*lambda_psi*w_0psi])];

T_inv = diag([1/T_x 1/T_y 1/T_psi]);

A = cs.m - cs.X_udot;
C = cs.m - cs.Y_vdot;
D = cs.m*cs.x_G - cs.Y_rdot;
F = cs.m*cs.x_G - cs.N_vdot;
G = cs.I_z - cs.N_rdot;
L = C*G - D*F;

for k = 1:n_s
psi = x(3,k);
u = x(4,k);
v = x(5,k);
r = x(6,k);
x_cdot = x(7,k);
y_cdot = x(8,k);
xi = x(9:14,k);
b = x(15:17,k);

% Rotasjonsmatrise
c_psi = cos(psi);
s_psi = sin(psi);

u_c = c_psi*x_cdot + s_psi*y_cdot; 
v_c = -s_psi*x_cdot + c_psi*y_cdot;

u_cdot = r*v_c;
v_cdot = -r*u_c;

% Relative speed
u_r = u - u_c;
v_r = v - v_c;

% Deriverte
u_dot = (tau(1) - cs.X_udot*u_cdot + cs.m*(cs.x_G*r + v)*r - (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)/2*r)*r...
         + (cs.X_u + cs.X_uu*abs(u_r) + cs.X_uuu*u_r^2)*u_r + c_psi*b(1) + s_psi*b(2))/A;  
     


E = (tau(2) - cs.Y_vdot*v_cdot - cs.m*u*r + cs.X_udot*u_r*r + cs.Y_r*r + cs.Y_v*v_r...
     + (cs.Y_vv*abs(v_r) + cs.Y_rv*abs(r))*v_r + (cs.Y_vr*abs(v_r) + cs.Y_rr*abs(r))*r... 
     - s_psi*b(1) + c_psi*b(2));
          
H = (tau(3) - cs.N_vdot*v_cdot - cs.m*cs.x_G*r*u + (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)/2*r)*u_r...
     - cs.X_udot*u_r*v_r + cs.N_v*v_r + cs.N_r*r + (cs.N_vv*abs(v_r) + cs.N_rv*abs(r))*v_r...
     + (cs.N_vr*abs(v_r) + cs.N_rr*abs(r))*r + b(3));
          
v_dot = (G*E-D*H)/L;
r_dot = (C*H-F*E)/L;

x_dot = c_psi*u - s_psi*v;
y_dot = s_psi*u + c_psi*v;
psi_dot = r;

xi_dot = Aw*xi;

b_dot = -T_inv*b;

mu_x = 0.05;
mu_y = 0.05;

z_dot(:,k) = [x_dot; y_dot; psi_dot; u_dot; v_dot; r_dot; -mu_x*x_cdot; -mu_y*y_cdot; xi_dot; b_dot];
end
end