function z_dot = ffun_surge(x,tau,d)
% ODE-function implementing Cybership II model. Used for estimating states
% with given input and current. Assumes 2. order modulus damping model.
% 
%
% x_dot = ffun_s(x,tau,d)
%
% INPUTS:
% x - complete state of ship. 
%   x(1:3) = eta  
%   x(4:6) = nu
% d(1:2) = [x_cdot; y_cdot]
% d(3:5) = [b_x; b_y; b_psi]
% tau - force input
%
% OUTPUT:
% x_dot - the value of differential equation

global cs

A = cs.m - cs.X_udot;

% Current is identical for all parallells
x_cdot = d(1);
y_cdot = d(2);

psi = x(3,k);
u = x(4,k);
v = x(5,k);
r = x(6,k);

% Rotasjonsmatrise
c_psi = cos(psi);
s_psi = sin(psi);

u_c = c_psi*x_cdot + s_psi*y_cdot;
v_c = -s_psi*x_cdot + c_psi*y_cdot;
      
u_cdot = r*v_c;

% Relative speed
u_r = u - u_c;
v_r = v - v_c;

% Deriverte
u_dot = (tau(1) - cs.X_udot*u_cdot + cs.m*(cs.x_G*r + v)*r - (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)/2*r)*r...
                  + (cs.X_u + cs.X_uu*abs(u_r) + cs.X_uuu*u_r^2)*u_r + c_psi*b_x + s_psi*b_y)/A;  

          
x_dot = c_psi*u - s_psi*v;


z_dot = [x_dot; u_dot;];
end

  
        
        