function z_dot = gfun_abko_p(x,tau,d)
% ODE-function implementing Cybership II model. Dual estimate parameter
% estimation.
%
% x_dot = gfun_abko_p_full(x,tau,d)
%
% INPUTS:
% d - Parameters
%   d(1:4) = abkowitz parameters
% tau - force input
%  tau(1:3) = controller input
%  tau(4:5)  = [x_cdot; y_cdot]
%  tau(6:8) = estimated bias
% x - Full vessel state
%   z(1:3)  = eta
%   z(4:6)  = nu
%
% OUTPUT:
% z_dot - the value of differential equation, based on parameters


global cs
n_s = size(d,2);

D_L = [-cs.X2_u 0 0;
    0 -cs.Y2_v -cs.Y2_r;
    0 -cs.N2_v -cs.N2_r];

z_dot = zeros(6,n_s);
psi = x(3);
u = x(4);
v = x(5);
r = x(6);
x_cdot = tau(4);
y_cdot = tau(5);
b = [tau(6); tau(7); tau(8)];

nu = [u; v; r;];

% Rotasjonsmatrise
c_psi = cos(psi);
s_psi = sin(psi);

R_psi = [c_psi  -s_psi   0;
         s_psi   c_psi   0;
         0       0       1];
% Straum
eta_cdot = [x_cdot; y_cdot; 0];
nu_c = R_psi'*eta_cdot; % Current speed rotated from earth-fixed to body-fixed
nu_cdot = -r*cs.S*nu_c; % d(nu_cb)/dt = 0

% Relative speed
nu_r = nu - nu_c;
u_r = nu_r(1);
v_r = nu_r(2);

% Deriverte
C_A = [0 0 (cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)*r/2);
       0 0 -cs.X_udot*u_r;
       -(cs.Y_vdot*v_r + (cs.N_vdot + cs.Y_rdot)*r/2) cs.X_udot*u_r 0];

C_RB = [0 0 -cs.m*(cs.x_G*r + v);
        0 0 cs.m*u;
        cs.m*cs.x_G*r 0 0];
    
 a = tau(1:3) - (D_L + C_A)*nu_r - C_RB*nu + cs.M_A*nu_cdot + R_psi'*b;


for k = 1:n_s
    D_NL = [-cs.X2_uuu*u_r^2 0 0;
        0 -cs.Y2_vvv*v_r^2-d(1,k)*v_r*r -d(2,k)*v_r*r-cs.Y2_rrr*r^2;
        0 -cs.N2_vvv*v_r^2-d(3,k)*v_r*r -d(4,k)*v_r*r-cs.N2_rrr*r^2];

    nu_dot = cs.M_inv*(a + D_NL*nu_r);
    z_dot(:,k) = [R_psi*nu; nu_dot;];
end
%     keyboard
end