global cs
G = 9.80665;

% Downscaled
L_d = 1.255;
M_d = 23.8;
T_d = sqrt(L_d/G);


% Upscaled
L_u = 1.255*70;
M_u = 4200000;
T_u = sqrt(L_u/G);


model = 'upscaled';

switch model
    case 'downscaled'
        m = 23.800;
        x_G = 0.046;
        I_z = 1.760;
        
        X_udot = -2.0;
        Y_vdot = -10.0;
        Y_rdot = -0.0;
        N_vdot = -0.0;
        N_rdot = -1.0;

        % Linear damping
        X_u = -0.72253;
        Y_v = -0.88965;
        Y_r = -7.250;
        N_v = 0.03130;
        N_r = -1.900;

        % Nonlinear damping
        X_uu = -1.32742;
        X_uuu = -5.86643;

        Y_vv = -36.47827;
        Y_rv = -0.805;
        Y_vr = -0.845;
        Y_rr = -3.450;

        N_vv = 3.95645;
        N_rv = 0.130;
        N_vr = 0.080;
        N_rr = -0.750;

    case 'dimensionless'
        m = 23.800/M_d;
        x_G = 0.046/L_d;
        I_z = 1.760/(M_d*L_d^2);
        
        % M_A
        X_udot = -2.0/M_d;
        Y_vdot = -10.0/M_d;
        Y_rdot = -0.0/(M_d*L_d);
        N_vdot = -0.0/(M_d*L_d);
        N_rdot = -1.0/(M_d*L_d^2);

        % Linear damping
        X_u = -0.72253*T_d/M_d;
        Y_v = -0.88965*T_d/M_d;
        Y_r = -7.250*T_d/(M_d*L_d);
        N_v = 0.03130*T_d/(M_d*L_d);
        N_r = -1.900*T_d/(M_d*L_d^2);

        % Nonlinear damping
        X_uu = -1.32742*L_d/M_d;
        X_uuu = -5.86643*L_d^2/(M_d*T_d);

        Y_vv = -36.47827*L_d/M_d;
        Y_rv = -0.805/M_d;
        Y_vr = -0.845/M_d;
        Y_rr = -3.450/(M_d*L_d);

        N_vv = 3.95645/M_d;
        N_rv = 0.130/(M_d*L_d);
        N_vr = 0.080/(M_d*L_d);
        N_rr = -0.750/(M_d*L_d^2);

case 'upscaled'
    % Northern clipper
        m = 23.800/M_d*M_u;
        x_G = 0.046/L_d*L_u;
        I_z = 1.760/(M_d*L_d^2)*(M_u*L_u^2);
        
        % M_A
        X_udot = -2.0/M_d*M_u;
        Y_vdot = -10.0/M_d*M_u;
        Y_rdot = -0.0/(M_d*L_d)*(M_u*L_u);
        N_vdot = -0.0/(M_d*L_d)*(M_u*L_u);
        N_rdot = -1.0/(M_d*L_d^2)*(M_u*L_u^2);

        % Linear damping - 2. Order Modulus
        X_u = -0.72253*T_d/M_d*M_u/T_u;
        Y_v = -0.88965*T_d/M_d*M_u/T_u;
        Y_r = -7.250*T_d/(M_d*L_d)*(M_u*L_u)/T_u;
        N_v = 0.03130*T_d/(M_d*L_d)*(M_u*L_u)/T_u;
        N_r = -1.900*T_d/(M_d*L_d^2)*(M_u*L_u^2)/T_u;

        % Nonlinear damping - 2. Order Modulus
        X_uu = -1.32742*L_d/M_d*M_u/L_u;
        X_uuu = -5.86643*L_d^2/(M_d*T_d)/L_u^2*(M_u*T_u);

        Y_vv = -36.47827*L_d/M_d*M_u/L_u;
        Y_rv = -0.805/M_d*M_u;
        Y_vr = -0.845/M_d*M_u;
        Y_rr = -3.450/(M_d*L_d)*(M_u*L_u);

        N_vv = 3.95645/M_d*M_u;
        N_rv = 0.130/(M_d*L_d)*(M_u*L_u);
        N_vr = 0.080/(M_d*L_d)*(M_u*L_u);
        N_rr = -0.750/(M_d*L_d^2)*(M_u*L_u^2);
        
        % Linear damping - Abkowitz
        X2_u = -2.1602e+004;
        Y2_v = -1.9360e+005;
        Y2_r = -1.1693e+007;
        N2_v = -1.9324e+008;
        N2_r = -2.1142e+008;

        % Nonlinear damping - Abkowitz
        X2_uuu = -2.1180e+003;

        Y2_vvv = -9.6265e+003;
        Y2_vvr = -0.805/M_d*M_u/2;
        Y2_vrr = -0.845/M_d*M_u/2;
        Y2_rrr = -3.9464e+008;

        N2_vvv = 0;
        N2_vvr = 0.130/(M_d*L_d)*(M_u*L_u)/2;
        N2_vrr = 0.080/(M_d*L_d)*(M_u*L_u)/2;
        N2_rrr = -6.0054e+009;
        
        % Linear damping
        X3_u = -7.5336e+004;
        Y3_v = -5.9291e+005;
        Y3_r = -1.5348e+007;
        N3_v = 2.1795e+006;
        N3_r = -2.6703e+008;
        
end

M_RB = [m 0 0; 0 m m*x_G; 0 m*x_G I_z];

M_A = [-X_udot 0 0; 0 -Y_vdot -Y_rdot; 0 -N_vdot -N_rdot];

D_L = [-X_u 0 0; 0 -Y_v -Y_r; 0 -N_v -N_r];

M_inv = (M_A + M_RB)^-1;

S = [0 -1 0; 1 0 0; 0 0 0];

cs = struct('m',m,'x_G',x_G,'I_z',I_z,'X_udot',X_udot,'Y_vdot',Y_vdot,'Y_rdot',Y_rdot,...
    'N_vdot',N_vdot,'N_rdot',N_rdot,'X_u',X_u,'Y_v',Y_v,'Y_r',Y_r,'N_v',N_v,'N_r',N_r,...
    'X_uu',X_uu,'X_uuu',X_uuu,'Y_vv',Y_vv,'Y_vr',Y_vr,'Y_rv',Y_rv,'Y_rr',Y_rr,...
    'N_vv',N_vv,'N_vr',N_vr,'N_rv',N_rv,'N_rr',N_rr,'M_RB',M_RB,'M_A',M_A,'M_inv',M_inv,'D_L',D_L,...
    'X2_u',X2_u,'Y2_v',Y2_v,'Y2_r',Y2_r,'N2_v',N2_v,'N2_r',N2_r,'X2_uuu',X2_uuu,...
    'Y2_vvv',Y2_vvv,'Y2_rrr',Y2_rrr,'N2_vvv',N2_vvv,'N2_rrr',N2_rrr,...
    'Y2_vvr',Y2_vvr,'Y2_vrr',Y2_vrr,'N2_vvr',N2_vvr,'N2_vrr',N2_vrr,'S',S,...
    'X3_u',X3_u,'Y3_v',Y3_v,'Y3_r',Y3_r,'N3_v',N3_v,'N3_r',N3_r);

clear m x_G I_z X_udot Y_vdot Y_rdot N_vdot N_rdot X_u Y_v Y_r N_v N_r... 
      X_uu X_uuu Y_vv Y_vr Y_rv Y_rr N_vv N_vr N_rv N_rr...
      M_RB M_A M_inv D_L S...
      G L_d L_u M_d M_u T_d T_u model...
      X2_uuu Y2_vvv Y2_rrr N2_vvv N2_rrr X2_u Y2_v Y2_r N2_v N2_r...
      Y2_vvr Y2_vrr N2_vvr N2_vrr U3_u Y3_v Y3_r N3_v N3_r;
      
  