function ref = ref_generator(k,Prob,operation,Ts,plott)
% Generates trajectory for controller.
% ref = ref_generator(k,Prob,operation,Ts,plott)
%
% INPUT:
%
%
% OUTPUT:

n_p = Prob.controller.n_p;
switch operation
    case 'SK' % good
        eta_ref = [Prob.ref.x_d*ones(1,n_p);
            Prob.ref.y_d*ones(1,n_p);
            Prob.ref.psi_d*ones(1,n_p)];
        ref = eta_ref;

    case 'Ellipsis'
        a = Prob.ref.par_a;
        b = Prob.ref.par_b;
        speed = Prob.ref.speed;
        L = 2*max(a,b)*pi;
        n_r = L*speed/Ts; % size(r)

        r_step = 2*pi/n_r;
        r_0 = k*r_step - pi;
        r_end = r_0 + 2*pi - r_step;
        r = r_0:r_step:r_end; % lengda til r = tid

        ref = [a*cos(r(1:n_p)); b*sin(r(1:n_p)); atan2(sin(r(1:n_p)),cos(r(1:n_p))) + pi/2 ];

        for l = 1:n_p
            n_rounds = floor((k+l)/n_r);
            ref(3,l) = ref(3,l) + n_rounds*2*pi;
        end
        
    case 'Line'
        speed = Prob.ref.speed;
        eta_ref = [zeros(1,200); 0:Ts*speed:(200-1)*Ts*speed; -pi/2*ones(1,200)];
        ref = zeros(3,n_p);
        for l = 1:n_p
            if k+l < size(eta_ref,2) + 1;
                ref(:,l) = eta_ref(:,k+l-1);
            else
                ref(:,l) = eta_ref(:,end);
            end
        end


    case 'Path'
        speed = Prob.ref.speed;
        eta_ref = zeros(Prob.simulation.n_y,Prob.simulation.samples+1);

        eta_ref(:,1:400) =      [zeros(1,400);                              0:4*Ts*speed:(400-1)*4*Ts*speed;                                zeros(1,400)];
        eta_ref(:,401:800) =    [0:2*Ts*speed:(400-1)*2*Ts*speed;           400*4*speed*Ts*ones(1,400);                                     -90*pi/180*ones(1,400)];
        eta_ref(:,801:1200) =   [(400)*2*Ts*speed*ones(1,400);              400*4*speed*Ts:Ts*speed:400*4*speed*Ts+(400-1)*Ts*speed;        -90*pi/180:270*pi/180/400:180*pi/180-270*pi/180/400];
        eta_ref(:,1201:1600) =  [(400)*2*Ts*speed:-2*Ts*speed:2*Ts*speed;   400*5*speed*Ts*ones(1,400);                                     180*pi/180*ones(1,400)];
        eta_ref(:,1601:2000) =  [zeros(1,400);                              400*5*speed*Ts:2*Ts*speed:400*5*speed*Ts+(400-1)*2*Ts*speed;    180*pi/180*ones(1,400)];
        eta_ref(:,2001) = [0; 400*7*speed*Ts; 180*pi/180];
        eta_ref(3,:) = eta_ref(3,:) + 90*pi/180;


        nu_ref = zeros(Prob.simulation.n_y,Prob.simulation.samples+1);
        nu_ref(:,1:400) =      [zeros(1,400);               4*Ts*speed*ones(1,400);     zeros(1,400)];
        nu_ref(:,401:800) =    [2*Ts*speed*ones(1,400);     zeros(1,400);               zeros(1,400)];
        nu_ref(:,801:1200) =   [zeros(1,400);               Ts*speed*ones(1,400);       270*pi/180/400*ones(1,400)];
        nu_ref(:,1201:1600) =  [-2*Ts*speed*ones(1,400);    zeros(1,400);               zeros(1,400)];
        nu_ref(:,1601:2000) =  [zeros(1,400);               2*Ts*speed*ones(1,400);     zeros(1,400)];
        nu_ref(:,2001) = [0; 0; 0];

        ref = zeros(6,n_p);
        for l = 1:n_p
            if k+l < size(nu_ref,2) + 1;
                ref(:,l) = [eta_ref(:,k+l-1); nu_ref(:,k+l-1)];
            else
                ref(:,l) = [eta_ref(:,end); nu_ref(:,end)];
            end
        end

    case 'Halfpath'
        speed = Prob.ref.speed;
        eta_ref = zeros(Prob.simulation.n_y,Prob.simulation.samples+1);

        eta_ref(:,1:200) =      [zeros(1,200);                              0:4*Ts*speed:(200-1)*4*Ts*speed;                                zeros(1,200)];
        eta_ref(:,201:400) =    [0:2*Ts*speed:(200-1)*2*Ts*speed;           200*4*speed*Ts*ones(1,200);                                     -90*pi/180*ones(1,200)];
        eta_ref(:,401:600) =    [(200)*2*Ts*speed*ones(1,200);              200*4*speed*Ts:Ts*speed:200*4*speed*Ts+(200-1)*Ts*speed;        -90*pi/180:270*pi/180/200:180*pi/180-270*pi/180/200];
        eta_ref(:,601:800) =    [(200)*2*Ts*speed:-2*Ts*speed:2*Ts*speed;   200*5*speed*Ts*ones(1,200);                                     180*pi/180*ones(1,200)];
        eta_ref(:,801:1000) =   [zeros(1,200);                              200*5*speed*Ts:2*Ts*speed:200*5*speed*Ts+(200-1)*2*Ts*speed;    180*pi/180*ones(1,200)];
        eta_ref(:,1001) = [0; 200*7*speed*Ts; 180*pi/180];
        eta_ref(3,:) = eta_ref(3,:) + 90*pi/180;


        nu_ref = zeros(Prob.simulation.n_y,Prob.simulation.samples+1);
        nu_ref(:,1:200) =    [zeros(1,200);               4*Ts*speed*ones(1,200);     zeros(1,200)];
        nu_ref(:,201:400) =  [2*Ts*speed*ones(1,200);     zeros(1,200);               zeros(1,200)];
        nu_ref(:,401:600) =  [zeros(1,200);               Ts*speed*ones(1,200);       270*pi/180/200*ones(1,200)];
        nu_ref(:,601:800) =  [-2*Ts*speed*ones(1,200);    zeros(1,200);               zeros(1,200)];
        nu_ref(:,801:1000) = [zeros(1,200);               2*Ts*speed*ones(1,200);     zeros(1,200)];
        nu_ref(:,1001) = [0; 0; 0];

        ref = zeros(6,n_p);
        for l = 1:n_p
            if k+l < size(nu_ref,2) + 1;
                ref(:,l) = [eta_ref(:,k+l-1); nu_ref(:,k+l-1)];
            else
                ref(:,l) = [eta_ref(:,end); nu_ref(:,end)];
            end
        end

end

if plott
    figure;
    hold on;
    plot(ref(2,:),ref(1,:),'b-');

    a = get(gca);
    points = 1:75:size(ref,2);
    axes_range_y = a.Position(3); % breidde
    axes_range_x = a.Position(4); % hgde
    range_y = a.XLim(end)-a.XLim(1); % span p x-aksen
    range_x = a.YLim(end)-a.YLim(1); % span p y-aksen

    arrow_x_start = ref(1,points); % Nord koordinat
    arrow_y_start = ref(2,points); % st koordinat
    arrow_y_end = arrow_y_start + sin(ref(3,points))*0.1*axes_range_x;
    arrow_x_end = arrow_x_start + cos(ref(3,points))*0.1*axes_range_y;
    axes_arrow_y_start = a.Position(1) + (arrow_y_start - a.XLim(1))*axes_range_y/range_y;
    axes_arrow_x_start = a.Position(2) + (arrow_x_start - a.YLim(1))*axes_range_x/range_x;
    axes_arrow_y_end = a.Position(1) + (arrow_y_end - a.XLim(1))*axes_range_y/range_y;
    axes_arrow_x_end = a.Position(2) + (arrow_x_end - a.YLim(1))*axes_range_x/range_x;

    for k = 1:length(arrow_x_start)-1
        % checks if arrow is contained in plot axes
        if arrow_y_start(:,k) >= a.XTick(1) && arrow_y_end(:,k) <= a.XTick(end) && arrow_x_start(:,k) >= a.YTick(1) && arrow_x_end(:,k) <= a.YTick(end)
            % draws arrow
            annotation('arrow',[axes_arrow_y_start(1,k); axes_arrow_y_end(1,k)],[axes_arrow_x_start(1,k); axes_arrow_x_end(1,k)]);
        end
    end
end
end
