function result = simulate(Prob)

%% %% %% Read variables

%% General
%Functions
integrator = Prob.simulation.integrator; ffun = Prob.simulation.ffun;

% Variables
Ts = Prob.simulation.Ts; samples = Prob.simulation.samples;
n_x = Prob.simulation.n_x; n_u = Prob.simulation.n_u;
n_y = Prob.simulation.n_y; n_p = Prob.controller.n_p;

result.x_sim = zeros(n_x,samples);
result.x_sim(:,1) = [Prob.simulation.eta_0; Prob.simulation.nu_0];
result.y_sim_pose = zeros(n_y,samples); 
result.y_sim_full = zeros(n_x,samples);
result.tau = zeros(n_u,samples);
t = Prob.simulation.t;
result.t = t;


%% Disturbances
% General additive noise
s_w = 1000; randn('seed',s_w); % always same randoms
w_noise = diag([Prob.disturbances.w_c_cov])*randn(6,samples); % Process noise

s_v = 10000; randn('seed',s_v); % always same randoms
v_noise = diag([Prob.disturbances.v_c_cov])*randn(6,samples); % Measurement noise

%% Variable initialization
% Current
if strcmp(Prob.disturbances.CURRENT,'On');
    dfun = Prob.disturbances.dfun;
    Tbvv = Prob.disturbances.Tbvv;
    Tbbb = Prob.disturbances.Tbbb;
    s_c = 0;
    randn('seed',s_c); % always same randoms
    c_noise = diag([Prob.disturbances.V_c_cov; Prob.disturbances.beta_c_cov])*randn(2,samples)/Ts; %gaussian % current noise
end
V_c_0 = Prob.disturbances.V_c_0;beta_c_0 = Prob.disturbances.beta_c_0;
result.d_sim = [ones(1,samples)*V_c_0; ones(1,samples)*beta_c_0];
result.d_sim_ned = zeros(2,samples); % Must be calculated on-line

% Waves
if strcmp(Prob.disturbances.WAVES,'On');
    waveforces = Prob.disturbances.waveforces; % Function
    wavedir_0 = Prob.disturbances.wavedir_0; % Main wave-direction
    shortcrest = Prob.disturbances.shortcrest; % Spreading on/off
    DOF = Prob.disturbances.DOF; % 3DOF or 6DOF waveforces
    tFinal = Prob.disturbances.tFinal; % N_waves ~~ 2*tFinal/pi;
end
% Inputs
waveForceTotal = zeros(n_u,samples);
waveForceDrift = zeros(n_u,samples);
waveForceFK = zeros(n_u,samples);
u = zeros(n_u*n_p,samples);
Performance_deviation = 0;
Performance_total_force = 0;
Performance_change_force = 0;
Prob.controller.u = zeros(n_u,samples);

result.ref = zeros(Prob.ref.ref_dimension,Prob.controller.n_p,samples);
result.ref_sim = zeros(Prob.ref.ref_dimension,samples);

%% Simulations
for k = 1:samples
    Prob.simulation.k = k;

    %% Update current, GOOD
    if strcmp(Prob.disturbances.CURRENT,'On')
        if k > 1; result.d_sim(:,k) = feval(integrator,dfun,result.d_sim(:,k-1),[Tbvv; Tbbb],[V_c_0; beta_c_0],c_noise(:,k-1),Ts); end
    end
    result.d_sim_ned(:,k) = [result.d_sim(1,k)*cos(result.d_sim(2,k)); result.d_sim(1,k)*sin(result.d_sim(2,k))];

    %% Simulate, GOOD!
    if k > 1
        result.x_sim(:,k) = feval(integrator,ffun,result.x_sim(:,k-1),result.tau(:,k-1),[result.d_sim_ned(:,k); zeros(3,1)],w_noise(:,k-1),Ts);
    end
    result.y_sim_pose(:,k) = hfun_pose(result.x_sim(:,k),v_noise(1:3,k));
    result.y_sim_full(:,k) = hfun_full(result.x_sim(:,k),v_noise(:,k));
       

    %% State & Disturbance Estimation UKF, good
    if strcmp(Prob.estimation.UKF_ONLINE,'On'); UKF_estimation_online; end
    %% Parameter estimation
    if strcmp(Prob.estimation.PE_ONLINE,'On'); PE_estimation_online; end

    %% Calculate trajectory
    result.ref(:,:,k) = feval(Prob.ref.ref_generator,k,Prob,Prob.ref.operation,Ts,0); % Last 1/0 is for plotting

%     Prob.ref.ref_current = result.ref(:,2,k);
    result.ref_sim(:,k) = result.ref(:,1,k);
    
    % Calculated forces and moments are applied during next iteration
    %% Update waveforces
    if strcmp(Prob.disturbances.WAVES,'On')
        [waveForceTotal(:,k),waveForceDrift(:,k),waveForceFK(:,k)] = feval(waveforces,t(k),rad2pipi(result.x_sim(3,k)-wavedir_0),0,DOF,shortcrest,tFinal);
        if strcmp(Prob.disturbances.WAVEDRIFT,'On')
            result.tau(:,k) = waveForceFK(:,k);% + waveForceDrift(:,k);
        else
            result.tau(:,k) = waveForceFK(:,k);% + waveForceDrift(:,k);
        end
    end


        %% Calculate next control input, unreliable
        if strcmp(Prob.controller.TYPE,'MPC')
            MPC_controller;
        elseif strcmp(Prob.controller.TYPE,'PID')% PID-control, good
            PID_controller;
        end
        result.u(:,k) = u(1:n_u,1);
        
%         calc_error = result.z_predicted(1:Prob.ref.ref_dimension,:,k) - result.ref(1:Prob.ref.ref_dimension,:,k)

        result.tau(:,k) = result.tau(:,k) + result.u(:,k); % waveforce + controller force

        % Calculate performance
        if k == 1; u_change = result.u(1:n_u,k);  n_r = Prob.ref.ref_dimension; Q = Prob.ref.Q;
        else u_change = result.u(1:n_u,k) - result.u(1:n_u,k-1); end

        Performance_deviation = Performance_deviation + (result.x_sim(1:n_r,k)-result.ref_sim(:,k))'*Q*(result.x_sim(1:n_r,k)-result.ref_sim(:,k));
        Performance_total_force = Performance_total_force + result.u(1:n_u,k)'*eye(3)*result.u(1:n_u,k);
        Performance_change_force = Performance_change_force + u_change'*eye(3)*u_change;
    end

    %% Store result variables
    result.Performance_deviation = Performance_deviation;
    result.Performance_total_force = Performance_total_force;
    result.Performance_change_force = Performance_change_force;

    if strcmp(Prob.disturbances.WAVES,'On')
        result.waveForceTotal = waveForceTotal;
        result.waveForceDrift = waveForceDrift;
        result.waveForceFK = waveForceFK;
    end

    if strcmp(Prob.controller.TYPE,'MPC')
        result.MPC_time = MPC_time;
    elseif strcmp(Prob.controller.TYPE,'PID')
        result.nu_lp = nu_lp;
    end

end