function [x_est,X_est] = ekf(integrator,ffun,hfun,ffun_jacobian,hfun_jacobian,constraints,measurement,estimates,input,d,Px,Rv,Rn,n_sp,Ts)
% UKF additive noise implementation. 
%
% [x_est,X_est] =
% ekf(@integrator,@ffun,@hfun,@ffun_jacobian,@hfun_jacobian,@constraints,measurement,estimates,input,Px,Rv,Rn,Ts)
%
% INPUTS:
% @integrator - function handle of the integrator method, @runge_kutta_4
% or @forward_euler
% @ffun - function handle, derivative function
% @hfun - function handle, measurement function
% @ffun_jacobian - function handle, derivative function
% @hfun_jacobian - function handle, measurement function
% @constraints - function handle of  integrator method, 
% measurement - Available measurements from time k, i.e. filtering.
% estimates - Previous estimate
% input - Input to system
% d - disturbances (known)
% Px - State covariance matrix
% Rv - Process noise covariance matrix
% Rn - Measurement noise covariance matrix
% Ts - integration time
%
% OUTPUT:
% x_est - State estimate
% X_est - State Covariance matrix
%% EKF
%     Tresh = 10^-6*ones(size(Result.x_k));
%     Fz = jac_compare(d(:,k),Result.x_k);
%     
%     [A,Fac] =
%     numjac(@jac_compare,d(:,k),Result.x_k,Fz,Tresh,FAC,VECTORIZED);
global FAC


[F,L] = ffun_jacobian(estimates,input,0);

VECTORIZED = 0;
Tresh = 10^-12*ones(size(estimates));
Fz = ffun_sdw2(input,estimates);
[F2,FAC] = numjac(@ffun_sdw2,input,estimates,Fz,Tresh,FAC,VECTORIZED);


H = hfun_jacobian(estimates,0);
D = eye(size(measurement,1));

P_apriori = F2*Px*F2' + L*Rv*L'; 
x_apriori = feval(integrator,ffun,estimates,input,d,0,Ts);
x_apriori = constraints(x_apriori);

% if rank(obsv(F,H)) == size(F,1)
%     disp('Observerbart')
% else
%     disp('Ikkje observerbart')
%     rank(obsv(F,H))
% end

K = P_apriori*H'*(H*P_apriori*H' + D*Rn*D')^-1;

m1 = K*H;
m2 = eye(size(m1)) - m1; % I - K*H
P_aposteriori = m2*P_apriori*m2' + K*Rn*K';
y_apriori = hfun(x_apriori,0);
x_aposteriori = x_apriori + K*(measurement - y_apriori);

%% 
X_est = P_aposteriori;
x_est = x_aposteriori;