function [S,Wc,Wm] = sigmapoints(mu,P,a,b,k,varargin)

% Finn dei n_s = 2*n_e+1 sigmapunkta til x med tilhyrande vekter for vanlig UKF
%
% [S,Wc,Wm] = sigmapoints(mu,P,varargin)
% S = sigmapunkta
% Wc = Vekting
% Wm = Vekting
% varargin{1} = alpha - skaleringsfaktor, typisk 1e-4 < alpha < 1
% varargin{2} = beta - tuneparameter, 2 optimalt for gaussisk
% varargin{3} - kappa - tuneparameter, ekstra spreiing til sigmapunkta typisk
% varargin{4} - number of 
% kappa = 0;
% mu - middelverdiane Lx1
% P - varians LxL

persistent A;

L = size(P,1); % n_e
n_x = size(mu,1);

%% sjekksett alpha
%0 < alpha < 1
if nargin < 2
    error('At least dimensionality n required.');
end
if nargin < 3
    a = [];
end
if nargin < 4
    b = [];
end
if nargin < 5
    k = [];
end

%
% Apply default values
%
if isempty(a)
    a = 0.05;
end
if isempty(b)
    b = 2;
end
if isempty(k)
    k = 0;
end

%% Konstantar
lambda =  a^2*(L+k)-L;

% legg til null for sty
mu_e = zeros(L,1);
mu_e(1:n_x,1) = mu;


%% Vektingsmatriser
Wc = zeros(1,2*L+1);
Wm = zeros(1,2*L+1);
Wc(1) = lambda/(L+lambda) + (1 - a^2 + b);
Wm(1) = lambda/(L+lambda);
Wc(2:2*L+1) = 1/(2*(L+lambda));
Wm(2:2*L+1) = Wc(2:2*L+1);


%% Punktverdiar
% Frste kovariansmatrise m vere positiv definitt

[C,p] = chol(P,'lower');
if nargin > 5
    if p == 0
        A{varargin{1}} = C;
    end
    B = [zeros(size(mu_e)) A{varargin{1}} -A{varargin{1}}];
    S = sqrt(L+lambda)*B + repmat(mu_e,1,2*L+1);
else
    if p == 0
        A = C;
    end
    B = [zeros(size(mu_e)) A -A];
    S = sqrt(L+lambda)*B + repmat(mu_e,1,2*L+1);
end




