function [x_est,X_est] = ukf_an(integrator,ffun,hfun,constraints,measurement,estimates,input,d,Px,Q,R,n_sp,Ts)
% UKF additive noise implementation. 
%
% [x_est,X_est] =
% ukf_an(@integrator,@ffun,@hfun,@constraints,measurement,estimates,input,Px,Rv,Rn,Ts)
%
% INPUTS:
% @integrator - function handle of the integrator method, @runge_kutta_4
% or @forward_euler
% @ffun - function handle of the system's derivative function
% @hfun - function handle of the system's measurement function
% @constraints - function handle of  integrator method, 
% measurement - Available measurements from time k, i.e. filtering.
% estimates - Previous estimate
% input - Input to system
% Px - State covariance matrix
% Rv - Process noise covariance matrix
% Rn - Measurement noise covariance matrix
% Ts - integration time
%
% OUTPUT:
% x_est - State estimate
% X_est - State Covariance matrix
%% additive noise ukf

n_y = size(measurement,1); % mledimensjon
n_x = size(estimates,1); % tilstandsdimensjon
n_s = 2*n_x+1; % Number of sigmapoints

[x_sigma,Wc,Wm] = sigmapoints(estimates,Px,0.1,0,2,n_sp); % 3.170
x_sigma = feval(constraints,x_sigma);

% Tidsoppdatering
x_sigma_neste = feval(integrator,ffun,x_sigma,input,d,0,Ts); % 3.171 

x_min = zeros(n_x,1);
for l = 1:n_s
    x_min = x_min + Wm(l)*x_sigma_neste(:,l); % 3.172
end

P_xmin = Q; % Process noise
for l=1:n_s
    P_xmin = P_xmin + ...
            Wc(l)*(x_sigma_neste(:,l)-x_min)*(x_sigma_neste(:,l)-x_min)'; % 3.173
end

% Redraw sigma-points
[x_sigma2,Wc2,Wm2] = sigmapoints(x_min,P_xmin,0.1,0,2,n_sp);
x_sigma2 = feval(constraints,x_sigma2);


% Mleoppdatering
y_sigma = feval(hfun,x_sigma2,0); % 3.175 feval(hfun,x_sigma2,noise)

y_min = zeros(n_y,1);
for l=1:n_s
    y_min = y_min + Wm2(l)*y_sigma(:,l); % 3.176
end

P_ymin = R; % Measurement noise
for l=1:n_s
    P_ymin = P_ymin + Wc2(l)*(y_sigma(:,l)-y_min)*(y_sigma(:,l)-y_min)'; % 3.177
end


P_xy = zeros(n_x,n_y);
for l=1:n_s
    P_xy = P_xy + Wc2(l)*(x_sigma2(:,l)-x_min)*(y_sigma(:,l)-y_min)'; % 3.178
end

% keyboard

K = P_xy*(P_ymin)^-1; % 3.179
x_apos = x_min + K*(measurement-y_min); %% Estimert verdi 3.180
P_x = P_xmin - K*P_ymin*K'; % 3.181
%% Outputs
x_est = x_apos;
X_est = P_x;