function [x_est,X_est] = ukf_gn(integrator,ffun,hfun,constraints,measurement,estimates,input,d,Px,Rv,Rn,n_sp,Ts)
% UKF general noise implementation. 
%
% [x_est,X_est] =
% ukf_gn(@integrator,@ffun,@hfun,@constraints,measurement,estimates,input,Px,Rv,Rn,Ts)
%
% INPUTS:
% @integrator - function handle of the integrator method, see runge_kutta_4
% or forward_euler
% @ffun - function handle of the system's derivative function
% @hfun - function handle of the system's measurement function
% @constraints - function handle of  integrator method, 
% measurement - Available measurements from time k, i.e. filtering.
% estimates - Previous estimate
% input - Input to system
% Px - State covariance matrix
% Rv - Process noise covariance matrix
% Rn - Measurement noise covariance matrix
% Ts - integration time
%
% OUTPUT:
% x_est - State estimate
% X_est - State Covariance matrix
%% general noise ukf

n_y = size(measurement,1); % additiv mlesty, 
n_x = size(estimates,1); % 
n_v = size(Rv,1); % additiv prosessty 
n_e = n_x + n_v + n_y; % tilstandar dim(eta)=3 dim(nu)=3 dim(nu_c)=2
n_s = 2*n_e+1; % Number of sigmapoints
    
Pa = blkdiag(Px,Rv,Rn);

[xa_sigma,Wc,Wm] = sigmapoints(estimates,Pa,0.1,0,2,n_sp); % 3.19

x_sigma = xa_sigma(1:n_x,:);
v_sigma = xa_sigma(n_x + 1:n_x+n_v,:);
n_sigma = xa_sigma(n_x + n_v + 1:end,:);

% Tidsoppdatering
noise = v_sigma;
 
x_sigma_neste = feval(integrator,ffun,x_sigma,input,d,noise,Ts);% simuler
x_sigma_neste = feval(constraints,x_sigma_neste);

x_min = zeros(n_x,1);
for l = 1:n_s
    x_min = x_min + Wm(l)*x_sigma_neste(:,l); % 3.21
end

P_xmin = zeros(n_x,n_x);
for l=1:n_s
    P_xmin = P_xmin + ...
            Wc(l)*(x_sigma_neste(:,l)-x_min)*(x_sigma_neste(:,l)-x_min)'; % 3.22
end

% Mleoppdatering
y_sigma = feval(hfun,x_sigma_neste,n_sigma); % 3.23


y_min = zeros(n_y,1);
for l=1:n_s
    y_min = y_min + Wm(l)*y_sigma(:,l); % 3.24
end

P_ymin = zeros(n_y);
for l=1:n_s
    P_ymin = P_ymin + Wc(l)*(y_sigma(:,l)-y_min)*(y_sigma(:,l)-y_min)'; % 3.25
end


P_xy = zeros(n_x,n_y);
for l=1:n_s
    P_xy = P_xy + Wc(l)*(x_sigma_neste(:,l)-x_min)*(y_sigma(:,l)-y_min)'; % 3.26
end

K = P_xy*(P_ymin)^-1; % 3.27
x_apos = x_min + K*(measurement-y_min); %% Estimert verdi 3.28
P_x = P_xmin - K*P_ymin*K'; % 3.29

%% Outputs
x_est = x_apos;
X_est = P_x;