function [w_est,W_est] = ukf_pe(integrator,ffun,hfun,constraints,measurement,estimates,input,d,Pw,Q,R,n_sp,Ts)
% UKF additive noise implementation. 
%
% [w_est,W_est] =
% ukf_pe(@integrator,@ffun,@hfun,@constraints,measurement,estimates,input,Px,Rv,Rn,Ts)
%
% INPUTS:
% @integrator - function handle of the integrator method
% @ffun - function handle of the system's derivative function. Uses
% parameters w as sigmapoints and states as known (d)
% @hfun - function handle of the system's measurement function
% @constraints - function handle of  integrator method, 
% measurement - Available measurements from time k, i.e. filtering.
% estimates - Previous estimate
% input - Force input, known
% d - known values
%  d(1:3)  = eta
%  d(4:6)  = nu
%  d(7:8)  = [x_cdot; y_cdot]
%  d(9:11) = bias
% Q - Parameter covariance matrix
% R - Parameter noise covariance matrix
% Rn - Measurement noise covariance matrix
% n_sp - number to separate parallell ukf sigmapoint calculations
% Ts - integration time
%
% OUTPUT:
% x_est - State estimate
% X_est - State Covariance matrix
%% additive noise ukf

n_y = size(measurement,1); % mledimensjon
n_w = size(estimates,1); % dim(parameters)
n_s = 2*n_w+1; % Number of sigmapoints

% Tidsoppdatering
w_min = estimates; % 3.136
P_wmin = Pw + Q; % 3.137

[w_sigma,Wc,Wm] = sigmapoints(w_min,P_wmin,0.1,2,0,n_sp); % 3.138
w_sigma = feval(constraints,w_sigma);


% gfun
x_sigma = zeros(6,n_s);
for l = 1:n_s
x_sigma(:,l) = feval(integrator,ffun,d,input,w_sigma(:,l),0,Ts); % 0 = noise...
end

y_sigma = feval(hfun,x_sigma,0); %3.139 feval(hfun,x_sigma,noise)


d_min = zeros(n_y,1);
for l=1:n_s
    d_min = d_min + Wm(l)*y_sigma(:,l); % 3.140
end

P_d = R; % Measurement noise
for l=1:n_s
    P_d = P_d + Wc(l)*(y_sigma(:,l)-d_min)*(y_sigma(:,l)-d_min)'; % 3.142
end


P_wy = zeros(n_w,n_y);
for l=1:n_s
    P_wy = P_wy + Wc(l)*(w_sigma(:,l)-w_min)*(y_sigma(:,l)-d_min)'; % 3.143
end

K = P_wy*(P_d)^-1; % 3.1
w_apos = w_min + K*(measurement-d_min); %% Estimert verdi 3.180
P_w = P_wmin - K*P_d*K'; % 3.181
% keyboard
%% Outputs
w_est = w_apos;
W_est = P_w;