function Sspread = spreadingWaveSpec_quick(S,W,CHI,Chi0,n,plotSpectrum)
% SPREADINGWAVESPEC calculates the directional spectrum spreading for a
% wave spectrum S. Useful for simulating short-time wave induced forces
% when waves propagate in different directions with a dominant direction
% (spreading).
%
% Parameters:
%    S   - wave spectrum
%    W	 - Vector of wave frequencies [rad/s]
%   CHI  - Vector of wave directions
%   Chi0 - dominant wave propagation direction [rad]
%    n   - determines engery distribution around xi0 (higher = sharper) [-]

% Setup
nCHI = length(CHI);

a1 = 2^(2*n-1)*factorial(n)*factorial(n-1)/(pi*factorial(2*n-1));

Chim = CHI - Chi0;


M = zeros(nCHI,1);
a = find(abs(Chim) < pi/2);
M(a) = a1*cos(Chim(a)).^(2*n);

Sspread = M*S;


if plotSpectrum
    figure;
    surf(W',CHI'.*180/pi,Sspread);
    xlabel('\omega [rad/s]'); ylabel('\chi -\chi_0 [deg]'); zlabel('S(\omega,\chi) [ms^2]');
end