clear all;
close all;
% addpath c:/tomlab/optim
addpath ../Framework/Modellar
addpath ../Framework/Modellar/n'ot used m-files'/
addpath ../Framework/Integrasjon
addpath ../Framework/UKF
addpath ../Framework/Ymse
addpath ../Framework/Waveforces
addpath ../Framework/MPC
addpath ../Framework/Plot
addpath ../Framework/Simulation
clc;
init_cybershipII




%% General
n_x = 6;
n_y = 3;
n_u = 3;
n_d = 2;
n_w = 9;
Ts = 0.8;
samples = 1000;


Prob = struct;
Prob.SIMULATE = 'On';
Prob.name = 'PID, PEllipsis Current Offline PE ';
Prob.disturbances.CURRENT = 'On';
Prob.disturbances.WAVES = 'Off';
Prob.disturbances.WAVEDRIFT = 'Off';
Prob.ref.operation = 'PEllipsis'; % SK, Ellipsis, Path

Prob.controller.TYPE = 'PID';
Prob.estimation.UKF_ONLINE = 'Off';
Prob.estimation.UKF_OFFLINE = 'On';
Prob.estimation.EKF_OFFLINE = 'On';
Prob.estimation.PE_ONLINE = 'Off';
Prob.estimation.PE_OFFLINE = 'On';

Prob.estimation.d_0 = 0*[0.5*cos(pi/3); 0.5*sin(pi/3)];

% Estimated values used in controller, 
Prob.estimation.S_ESTIMATE_ONLINE = 'Off'; 
Prob.estimation.D_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC
Prob.estimation.B_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC

% Estimated values used in PE
Prob.estimation.S_ESTIMATE_OFFLINE = 'On'; 
Prob.estimation.D_ESTIMATE_OFFLINE = 'On';
Prob.estimation.B_ESTIMATE_OFFLINE = 'Off';

Prob.ref.operation = 'Ellipsis';
par_a = 40;
par_b = 40;
speed = 1;
settle_time = 400;

if strcmp(Prob.ref.operation,'PEllipsis') || strcmp(Prob.ref.operation,'PEllipsis')
    Prob.simulation.eta_0 = [-par_a; 0; -pi/2];
else
    Prob.simulation.eta_0 = [0; 0; 0];
end
Prob.simulation.nu_0 = zeros(n_u,1);



%% Functions, GOOD!
Prob.simulation.integrator = @runge_kutta_4; Prob.simulation.ffun = @ffun2_s;

%% Variables, GOOD!
Prob.simulation.Ts = Ts;Prob.simulation.samples = samples;Prob.simulation.n_x = n_x;
Prob.simulation.n_y = n_y;Prob.simulation.n_u = n_u;
Prob.simulation.n_w = n_w;Prob.simulation.n_d = n_d;
Prob.simulation.t = 0:Ts:(samples-1)*Ts;


%% Reference generator, GOOD!
Prob.ref.ref_generator = @ref_generator;
Prob.ref.par_a = par_a;
Prob.ref.par_b = par_a;
Prob.ref.speed = speed;
Prob.ref.settle_time = settle_time;
Prob.ref.Q = diag([5 5 5*(180/pi)^2]);
Prob.ref.R = 0*diag([1 1 0.01])*10^-6; 
Prob.ref.S = 0*diag([1 1 0.01])*10^-2;
Prob.ref.ref_dimension = size(Prob.ref.Q,1);



%% Disturbances, GOOD!
% General additive noise
Prob.disturbances.w_c_cov = zeros(n_x,1); % finn ekte verdiar
Prob.disturbances.v_c_cov = 0*[0.2^2 0.2^2 (0.025*pi/180)^2 0.10^2 0.10^2 (0.1*pi/180)^2]; % finn ekte verdiar

% Current
if strcmp(Prob.disturbances.CURRENT,'On')
    Prob.disturbances.dfun = @dfun_sim;
    Prob.disturbances.Tbvv = 500; Prob.disturbances.Tbbb = 500;
    Prob.disturbances.V_c_cov = 0.005; Prob.disturbances.beta_c_cov = pi/900;
    Prob.disturbances.V_c_0 = 0.5; Prob.disturbances.beta_c_0 = pi/3;
end

%%
if strcmp(Prob.estimation.UKF_OFFLINE,'On')
    Q_s = diag([0 0 0 1e-3 1e-3 1e-3]);
    Q_d = diag([0.001^2 0.001^2]);
    R_pose = diag([0.25^2 0.25^2 (0.15*pi/180)^2]);

    S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ukf = diag([0.5^2 0.5^2]);

    %% SD abko
    n_s = n_x + n_d;
    Prob.estimation.ukf_algorithm_offline{1} = @ukf_an;
    Prob.estimation.ukf_update_function_offline{1} = @ffun_abko_sd;
    Prob.estimation.ukf_measurement_function_offline{1} = @hfun_pose;
    Prob.estimation.ukf_integrator_offline{1} = @runge_kutta_4;
    Prob.estimation.ukf_constraints_offline{1} = @constraints_velocity_current;
    Prob.estimation.ukf_ns_offline{1} = n_s; % Number of estimated states
    Prob.estimation.ukf_ny_offline{1} = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ukf_nx_offline{1} = n_x;
    Prob.estimation.ukf_nd_offline{1} = n_d;
    Prob.estimation.ukf_parameters_offline{1} = 'result.p_ukf_abko(:,k-1)';

    % Sette kovariansar
    Prob.estimation.ukf_Q_offline{1} = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ukf_R_offline{1} = blkdiag(R_pose); % Measurement noise covariance
    Prob.estimation.X_ukf_0_offline{1} = blkdiag(S_ukf,D_ukf);
    
    Q_s = diag([0 0 0 0 0 0]);
    Q_d = diag([0.001^2 0.001^2]);
    R_pose = diag([0.25^2 0.25^2 (0.15*pi/180)^2]);

    S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ukf = diag([0.5^2 0.5^2]);
    
    %% SD modulus
    n_s = n_x + n_d;
    Prob.estimation.ukf_algorithm_offline{2} = @ukf_an;
    Prob.estimation.ukf_update_function_offline{2} = @ffun_sd;
    Prob.estimation.ukf_measurement_function_offline{2} = @hfun_pose;
    Prob.estimation.ukf_integrator_offline{2} = @runge_kutta_4;
    Prob.estimation.ukf_constraints_offline{2} = @constraints_velocity_current;
    Prob.estimation.ukf_ns_offline{2} = n_s; % Number of estimated states
    Prob.estimation.ukf_ny_offline{2} = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ukf_nx_offline{2} = n_x;
    Prob.estimation.ukf_nd_offline{2} = n_d;
    Prob.estimation.ukf_parameters_offline{2} = 'zeros(5,1)';

    % Sette kovariansar
    Prob.estimation.ukf_Q_offline{2} = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ukf_R_offline{2} = blkdiag(R_pose); % Measurement noise covariance
    Prob.estimation.X_ukf_0_offline{2} = blkdiag(S_ukf,D_ukf);
end
%% DUAL ESTIMATION
if strcmp(Prob.estimation.PE_ONLINE,'On') || strcmp(Prob.estimation.PE_OFFLINE,'On')
    Prob.estimation.pe_tuning = 'Constant';
    R_pose = diag([0.25^2 0.25^2 (0.25*pi/180)^2]);
    Prob.estimation.pe_algorithm = @ukf_pe;
    Prob.estimation.pe_states_function = @gfun_abko_p;
    Prob.estimation.pe_measurement_function = @hfun_pose;
    Prob.estimation.pe_integrator = @runge_kutta_4;
    Prob.estimation.pe_constraints = @constraints; % constraints empty

    Prob.estimation.measurements = strcat('result.y_sim_pose(:,k)');

    % Tuning
    Prob.estimation.pe_R = R_pose; % Process noise covariance
    Prob.estimation.pe_Q = diag([0.01^2 0.01^2 0.01^2 0.01^2;]);
    Prob.estimation.P_0 = diag([100000^2 100000^2 100000^2 100000^2]);
end
%% EKF, not GOOD, SD
if strcmp(Prob.estimation.EKF_OFFLINE,'On')
    Q_s_ekf = blkdiag(diag([0.2 0.2 0.2]),diag([3 3 (180/pi)^2])*10);
    Q_d_ekf = diag([15^2 15^2]);
    R_pose_ekf = diag([1^2 1^2 (1*pi/180)^2]);

    S_ekf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ekf = diag([2 2]);

    % Sette initialverdiar
    Prob.estimation.eta_0 = Prob.simulation.eta_0;
    Prob.estimation.nu_0 = zeros(3,1);

    %% EKF SDW, modulus
    n_s = n_x + n_d;
    Prob.estimation.ekf_algorithm_offline = @ekf;
    Prob.estimation.ekf_update_function_offline = @ffun_sd_ekf;
    Prob.estimation.ekf_measurement_function_offline = @hfun_pose;
    Prob.estimation.ekf_update_jacobian_offline = @ffun_sd_ekf_jacobian;
    Prob.estimation.ekf_measurement_jacobian_offline = @hfun_pose_jacobian;
    Prob.estimation.ekf_integrator_offline = @runge_kutta_4;
    Prob.estimation.ekf_constraints_offline = @constraints_velocity_current;
    Prob.estimation.ekf_ns_offline = n_s; % Number of estimated states
    Prob.estimation.ekf_ny_offline = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ekf_nx_offline = n_x;
    Prob.estimation.ekf_nd_offline = n_d;
    Prob.estimation.ekf_parameters_offline = 'zeros(5,1)';

    % Sette kovariansar
    Prob.estimation.ekf_Q_offline = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ekf_R_offline = R_pose; % Measurement noise covariance
    Prob.estimation.X_ekf_0_offline = blkdiag(S_ukf,D_ukf);
end


%% PID kontroller
if strcmp(Prob.controller.TYPE,'PID')
    Prob.controller.n_p = 2;
    wb_x = 0.25; wb_y = 0.20; wb_psi = 15*pi/180;
    eta_x = 0.790; eta_y = 0.790; eta_psi = 0.790;
    wn_x = wb_x/sqrt(1-2*eta_x^2 + sqrt(4*eta_x^4 - 4*eta_x^2 + 2));
    wn_y = wb_y/sqrt(1-2*eta_y^2 + sqrt(4*eta_y^4 - 4*eta_y^2 + 2));
    wn_psi = wb_psi/sqrt(1-2*eta_psi^2 + sqrt(4*eta_psi^4 - 4*eta_psi^2 + 2));
    m_x = cs.m+cs.X_udot; m_y = cs.m+cs.Y_vdot; m_psi = cs.I_z+cs.N_rdot;
    d_x = cs.X3_u; d_y = cs.Y3_v; d_psi = cs.N3_r;
    Km_x = 0; Km_y = 0; Km_psi = 0;
    Kp_x = (m_x + Km_x)*wn_x^2; Kp_y = (m_y + Km_y)*wn_y^2;  Kp_psi = (m_psi + Km_psi)*wn_psi^2;
    Kd_x = ((m_x + Km_x)*wn_x*2*eta_x - d_x); Kd_y = ((m_y + Km_y)*wn_y*2*eta_y - d_y);  Kd_psi = ((m_psi + Km_psi)*wn_psi*2*eta_psi - d_psi);
    Ki_x = wn_x/10*Kp_x; Ki_y = wn_y/10*Kp_y; Ki_psi = wn_psi/10*Kp_psi;

    Prob.controller.Kp = diag([Kp_x; Kp_y; Kp_psi]);
    Prob.controller.Kd = diag([Kd_x; Kd_y; Kd_psi]);
    Prob.controller.Ki = diag([Ki_x; Ki_y/2; Ki_psi/2]);
end
%%

if strcmp(Prob.SIMULATE,'On')
    result = simulate(Prob);
    save sim_result result
    
else
    load sim_result
end
%%

for k = 1:samples;
    if strcmp(Prob.estimation.UKF_OFFLINE,'On');  
        UKF_estimation_offline; 
    end

    if strcmp(Prob.estimation.EKF_OFFLINE,'On'); EKF_estimation_offline; end

    if strcmp(Prob.estimation.PE_OFFLINE,'On'); PE_estimation_offline; end
end
%%
plot_disturbances(result.t,[result.d_sim_ned; result.d_sim],'k-',1);
plot_disturbances(result.t,result.x_ukf_offline{1}(7:8,:),'b:',1);
plot_disturbances(result.t,result.x_ukf_offline{2}(7:8,:),'g--',1);
plot_disturbances(result.t,result.x_ekf_offline(7:8,:),'r-.',1);

plot_states_controller(result.t,result.ref_sim,'k-',3);
plot_states_controller(result.t,result.x_sim,'m-.',3);

plot_states_est(result.t,result.x_sim,'k-',4);
plot_states_est(result.t,result.x_ukf_offline{1}(1:6,:),'b:',4);
plot_states_est(result.t,result.x_ukf_offline{2}(1:6,:),'g--',4);
plot_states_est(result.t,result.x_ekf_offline(1:6,:),'r-.',4);

plot_parameters(result.t,result.p_ukf_abko,'k-',5);



