clear all;
close all;
% addpath c:/tomlab/optim
% addpatch c
addpath ../Framework/Modellar
addpath ../Framework/Integrasjon
addpath ../Framework/UKF
addpath ../Framework/Ymse
addpath ../Framework/Waveforces
addpath ../Framework/MPC
addpath ../Framework/Plot
addpath ../Framework/Simulation
clc;
init_cybershipII




%% General
n_x = 6;
n_y = 3;
n_u = 3;
n_d = 2;
n_w = 9;
Ts = 0.8;
samples = 250;


Prob = struct;
Prob.SIMULATE = 'Off';
Prob.name = 'MPC, PEllipsis Current Offline PE ';
Prob.disturbances.CURRENT = 'Off';
Prob.disturbances.WAVES = 'Off';
Prob.disturbances.WAVEDRIFT = 'Off';
Prob.ref.operation = 'Line'; % SK, Ellipsis, Path
Prob.ref.speed = 3;

Prob.controller.TYPE = 'MPC';
Prob.estimation.UKF_ONLINE = 'Off';
Prob.estimation.UKF_OFFLINE = 'Off';
Prob.estimation.EKF_OFFLINE = 'Off';
Prob.controller.CONSTRAINED = 'On';
Prob.estimation.PE_ONLINE = 'Off';
Prob.estimation.PE_OFFLINE = 'Off';

Prob.estimation.d_0 = 0*[0.5*cos(pi/3); 0.5*sin(pi/3)];

% Estimated values used in controller, 
Prob.estimation.S_ESTIMATE_ONLINE = 'Off'; 
Prob.estimation.D_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC
Prob.estimation.B_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC

% Estimated values used in PE
Prob.estimation.S_ESTIMATE_OFFLINE = 'Off'; 
Prob.estimation.D_ESTIMATE_OFFLINE = 'Off';
Prob.estimation.B_ESTIMATE_OFFLINE = 'Off';

%MPC
Prob.controller.CONSTRAINED = 'On';
n_p = 5;
n_c = 3;
n_o = 3;
n_b = 1;

Prob.controller.x_con = 2;
Prob.controller.y_con = 5;
Prob.controller.psi_con = pi/180;

    
Prob.simulation.eta_0 = [0; 0; -pi/2];
Prob.simulation.nu_0 = zeros(n_u,1);

Prob.disturbances.V_c_0 = 0; Prob.disturbances.beta_c_0 = pi/3;


%% Functions, GOOD!
Prob.simulation.integrator = @runge_kutta_4; Prob.simulation.ffun = @ffun_s;

%% Variables, GOOD!
Prob.simulation.Ts = Ts;Prob.simulation.samples = samples;Prob.simulation.n_x = n_x;
Prob.simulation.n_y = n_y;Prob.simulation.n_u = n_u;
Prob.simulation.n_w = n_w;Prob.simulation.n_d = n_d;
Prob.simulation.t = 0:Ts:(samples-1)*Ts;


%% Reference generator, GOOD!
Prob.ref.ref_generator = @ref_generator;
Prob.ref.Q = diag([5 5 5*(180/pi)^2]);
Prob.ref.R = 0*diag([1 1 0.01])*10^-6; 
Prob.ref.S = 0*diag([1 1 0.01])*10^-2;
Prob.ref.ref_dimension = size(Prob.ref.Q,1);

%% Disturbances, GOOD!
% General additive noise
Prob.disturbances.w_c_cov = zeros(n_x,1); % finn ekte verdiar
Prob.disturbances.v_c_cov = 0*[0.2^2 0.2^2 (0.025*pi/180)^2 0.10^2 0.10^2 (0.1*pi/180)^2]; % finn ekte verdiar

%% MPC Controller
if strcmp(Prob.controller.TYPE,'MPC')
    Prob.controller.n_p = n_p; % Prediction horizon
    Prob.controller.n_c = n_c; % Control horizon
    Prob.controller.n_b = n_b; % Blocking
    Prob.controller.n_o = n_o; % Different blocks

    Prob.controller.integrator = @runge_kutta_4;
    Prob.controller.system_update_function = @ffun_s_linear; % @ffun_abko_s % ffun_s_linear
    Prob.controller.options = optimset('Display','final','LargeScale','Off');
    
    Prob.controller.Q = blkdiag2(Prob.ref.Q,n_p);
    Prob.controller.R = blkdiag2(Prob.ref.R,n_p);
    Prob.controller.S = blkdiag2(Prob.ref.S,n_p-1);

    Prob.controller.optimization_function = @NMPC;
    if strcmp(Prob.controller.CONSTRAINED,'On')
        Prob.controller.A = [];
        Prob.controller.B = [];
        Prob.controller.Aeq = [];
        Prob.controller.Beq = [];
        Prob.controller.NONLCON = []; % state constraints typically
        Prob.controller.u_L = -repmat([1; 1; 10]*1e8,n_p,1);
        Prob.controller.u_U = repmat([1; 1; 10]*1e8,n_p,1);
    end   
end
%% PID kontroller
if strcmp(Prob.controller.TYPE,'PID')
    Prob.controller.n_p = 1;
    wb_x = 0.25; wb_y = 0.20; wb_psi = 15*pi/180;
    eta_x = 0.790; eta_y = 0.790; eta_psi = 0.790;
    wn_x = wb_x/sqrt(1-2*eta_x^2 + sqrt(4*eta_x^4 - 4*eta_x^2 + 2));
    wn_y = wb_y/sqrt(1-2*eta_y^2 + sqrt(4*eta_y^4 - 4*eta_y^2 + 2));
    wn_psi = wb_psi/sqrt(1-2*eta_psi^2 + sqrt(4*eta_psi^4 - 4*eta_psi^2 + 2));
    m_x = cs.m+cs.X_udot; m_y = cs.m+cs.Y_vdot; m_psi = cs.I_z+cs.N_rdot;
    d_x = cs.X3_u; d_y = cs.Y3_v; d_psi = cs.N3_r;
    Km_x = 0; Km_y = 0; Km_psi = 0;
    Kp_x = (m_x + Km_x)*wn_x^2; Kp_y = (m_y + Km_y)*wn_y^2;  Kp_psi = (m_psi + Km_psi)*wn_psi^2;
    Kd_x = ((m_x + Km_x)*wn_x*2*eta_x - d_x); Kd_y = ((m_y + Km_y)*wn_y*2*eta_y - d_y);  Kd_psi = ((m_psi + Km_psi)*wn_psi*2*eta_psi - d_psi);
    Ki_x = wn_x/10*Kp_x; Ki_y = wn_y/10*Kp_y; Ki_psi = wn_psi/10*Kp_psi;

    Prob.controller.Kp = diag([Kp_x; Kp_y; Kp_psi]);
    Prob.controller.Kd = diag([Kd_x; Kd_y; Kd_psi]);
    Prob.controller.Ki = diag([Ki_x; Ki_y/2; Ki_psi/2]);
end
%%

if strcmp(Prob.SIMULATE,'On')
    result = simulate(Prob);
    save sim_result result
else
    load sim_result
end
%%

for k = 1:samples;
    if strcmp(Prob.estimation.UKF_OFFLINE,'On');  
        UKF_estimation_offline; 
    end

    if strcmp(Prob.estimation.EKF_OFFLINE,'On'); EKF_estimation_offline; end

    if strcmp(Prob.estimation.PE_OFFLINE,'On'); PE_estimation_offline; end
end
%%
plot_disturbances(result.t,[result.d_sim_ned; result.d_sim; zeros(3,samples)],'k-',1);
% plot_disturbances(result.t,[result.x_ukf_offline{1}(7:8,:); zeros(3,samples)],'b:',1);

plot_states_controller(result.t,result.ref_sim,'k-',3);
plot_states_controller(result.t,result.x_sim,'m-.',3);

plot_states_est(result.t,result.x_sim,'k-',4);
% plot_states_est(result.t,result.x_ukf_offline{1}(1:6,:),'b:',4);






