clear all;
close all;
% addpath c:/tomlab/optim
addpath ../Framework/Modellar
addpath ../Framework/Modellar/n'ot used m-files'/
addpath ../Framework/Integrasjon
addpath ../Framework/UKF
addpath ../Framework/Ymse
addpath ../Framework/Waveforces
addpath ../Framework/MPC
addpath ../Framework/Plot
addpath ../Framework/Simulation
clc;
init_cybershipII



%% General
n_x = 6;
n_y = 3;
n_u = 3;
n_d = 2;
n_w = 9;
Ts = 0.8;
samples = 200;


Prob = struct;
Prob.SIMULATE = 'On';
Prob.name = 'MPC using UKF estimate of current and waves';
Prob.disturbances.CURRENT = 'On';
Prob.disturbances.WAVES = 'On'; %%%
Prob.disturbances.WAVEDRIFT = 'Off';
Prob.ref.operation = 'SK'; % SK, Ellipsis, Path
Prob.simulation.eta_0 = [-5; 2; -pi/2];
Prob.simulation.nu_0 = zeros(n_u,1);
Prob.controller.TYPE = 'MPC';
Prob.estimation.UKF_ONLINE = 'On';
Prob.estimation.UKF_OFFLINE = 'Off';
Prob.estimation.EKF_OFFLINE = 'Off';
Prob.controller.CONSTRAINED = 'On';
Prob.estimation.PE_ONLINE = 'Off';
Prob.estimation.PE_OFFLINE = 'Off';

Prob.estimation.d_0 = 0*[0.5*cos(pi/3); 0.5*sin(pi/3)];

Prob.controller.decoupled = 'On';

% Estimated values used in controller, 
Prob.estimation.S_ESTIMATE_ONLINE = 'On'; 
Prob.estimation.D_ESTIMATE_ONLINE = 'On'; % ONLY FOR MPC
Prob.estimation.B_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC

n_p = 12;
n_c = 10;
n_o = 10;
n_b = 1;

Prob.controller.x_con = 20;
Prob.controller.y_con = 20;
Prob.controller.psi_con = inf;

%% Functions, GOOD!
Prob.simulation.integrator = @runge_kutta_4; Prob.simulation.ffun = @ffun2_s;

%% Variables, GOOD!
Prob.simulation.Ts = Ts;Prob.simulation.samples = samples;Prob.simulation.n_x = n_x;
Prob.simulation.n_y = n_y;Prob.simulation.n_u = n_u;
Prob.simulation.n_w = n_w;Prob.simulation.n_d = n_d;
Prob.simulation.t = 0:Ts:(samples-1)*Ts;


%% Reference generator, GOOD!
Prob.ref.ref_generator = @ref_generator;

Prob.ref.x_d = 0; Prob.ref.y_d = 0; Prob.ref.psi_d = 0;
Prob.ref.Q = diag([5 5 (5*180/pi)^2]); %1 1 1*(180/pi)^2])*10^3; % Inkludere farta?
Prob.ref.R = zeros(3); 
Prob.ref.S = zeros(3); 
Prob.ref.ref_dimension = size(Prob.ref.Q,1);
%% Disturbances, GOOD!
% General additive noise
Prob.disturbances.w_c_cov = zeros(n_x,1); % finn ekte verdiar
Prob.disturbances.v_c_cov = 0*[0.2^2 0.2^2 (0.025*pi/180)^2 0.10^2 0.10^2 (0.1*pi/180)^2]; % finn ekte verdiar

% Current
if strcmp(Prob.disturbances.CURRENT,'On')
    Prob.disturbances.dfun = @dfun_sim;
    Prob.disturbances.Tbvv = 500; Prob.disturbances.Tbbb = 500;
    Prob.disturbances.V_c_cov = 0.005; Prob.disturbances.beta_c_cov = pi/900;
end
Prob.disturbances.V_c_0 = 0.5; Prob.disturbances.beta_c_0 = pi/3;
% Waves
if strcmp(Prob.disturbances.WAVES,'On')
    Prob.disturbances.waveforces = @waveForces; % Function
    Prob.disturbances.wavedir_0 = pi; % Main wave-direction
    Prob.disturbances.shortcrest = 0; % Spreading on/off
    Prob.disturbances.DOF = n_y; % 3DOF or 6DOF waveforces
    Prob.disturbances.tFinal = 60*pi/2; % N_waves ~~ 2*tFinal/pi;
end
%% Estimation
%UKF, not GOOD, SDW
if strcmp(Prob.estimation.UKF_ONLINE,'On')
    Q_s = diag([0 0 0 1e-8 1e-8 1e-10]);
    Q_d = diag([0.000015^2 0.000015^2]);
    R_pose = diag([0.25^2 0.25^2 (0.2*pi/180)^2]);
    S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ukf = diag([0.5^2 0.5^2]);

    % Sette initialverdiar
    Prob.estimation.eta_0 = Prob.simulation.eta_0;
    Prob.estimation.nu_0 = zeros(3,1);
    
    %% UKF SD, modulus
    n_s = n_x + n_d;
    Prob.estimation.ukf_algorithm_online = @ukf_an;
    Prob.estimation.ukf_update_function_online = @ffun_sd;
    Prob.estimation.ukf_measurement_function_online = @hfun_pose;
    Prob.estimation.ukf_integrator_online = @runge_kutta_4;
    Prob.estimation.ukf_constraints_online = @constraints_velocity_current;
    Prob.estimation.ukf_ns_online = n_s; % Number of estimated states
    Prob.estimation.ukf_ny_online = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ukf_nx_online = n_x;
    Prob.estimation.ukf_nd_online = n_d;
    Prob.estimation.ukf_parameters_online = 'zeros(5,1)';
    
    if strcmp(Prob.estimation.S_ESTIMATE_ONLINE,'On')
        Prob.estimation.s_estimate = 'result.x_ukf_online(1:6,k)';
    end
    if strcmp(Prob.estimation.D_ESTIMATE_ONLINE,'On')
        Prob.estimation.d_estimate = 'result.x_ukf_online(7:8,k)';
    end
    if strcmp(Prob.estimation.B_ESTIMATE_ONLINE,'On')
        Prob.estimation.b_estimate = 'result.x_ukf_online(15:17,k)';
    end

    % Sette kovariansar
    Prob.estimation.ukf_Q_online = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ukf_R_online = R_pose; % Measurement noise covariance
    Prob.estimation.X_ukf_0_online = blkdiag(S_ukf,D_ukf);
end
%% MPC Controller
if strcmp(Prob.controller.TYPE,'MPC')
    Prob.controller.n_p = n_p; % Prediction horizon
    Prob.controller.n_c = n_c; % Control horizon
    Prob.controller.n_b = n_b; % Blocking
    Prob.controller.n_o = n_o; % Different blocks

    Prob.controller.integrator = @runge_kutta_4;
    Prob.controller.system_update_function = @ffun_s_linear; % @ffun_abko_s % ffun_s_linear
    Prob.controller.options = optimset('Display','final','LargeScale','Off','MaxFunEval',20000);
    
    Prob.controller.Q = blkdiag2(Prob.ref.Q,n_p);
    Prob.controller.R = blkdiag2(Prob.ref.R,n_p);
    Prob.controller.S = blkdiag2(Prob.ref.S,n_p-1);

    Prob.controller.optimization_function = @NMPC;
    if strcmp(Prob.controller.CONSTRAINED,'On')
        Prob.controller.A = [];
        Prob.controller.B = [];
        Prob.controller.Aeq = [];
        Prob.controller.Beq = [];
        Prob.controller.NONLCON = @state_constraint; % state constraints typically
        Prob.controller.u_L = -repmat([1; 1; 10]*1e8,n_p,1);
        Prob.controller.u_U = repmat([1; 1; 10]*1e8,n_p,1);
    end   
end
%% SIMULATIONS

if strcmp(Prob.SIMULATE,'On')
    result = simulate(Prob);
    save sim_result result
else
    load sim_result
end
%%

for k = 1:samples;
    if strcmp(Prob.estimation.UKF_OFFLINE,'On');  
        UKF_estimation_offline; 
    end

    if strcmp(Prob.estimation.EKF_OFFLINE,'On'); EKF_estimation_offline; end

    if strcmp(Prob.estimation.PE_OFFLINE,'On'); PE_estimation_offline; end
end
%%
plot_disturbances(result.t,[result.d_sim_ned; result.d_sim; zeros(3,samples)],'k-',1);
plot_disturbances(result.t,[result.x_ukf_online(7:8,:); zeros(3,samples)],'b:',1);%result.x_ukf_online(15:17,:)

plot_states_controller(result.t,result.ref_sim,'k-',3);
plot_states_controller(result.t,result.x_sim,'m-.',3);

plot_states_est(result.t,result.x_sim,'k-',4);
plot_states_est(result.t,result.x_ukf_online(1:6,:),'b:',4);