clear all;
close all;
% addpath c:/tomlab/optim
addpath ../Framework/Modellar
addpath ../Framework/Modellar/n'ot used m-files'/
addpath ../Framework/Integrasjon
addpath ../Framework/UKF
addpath ../Framework/Ymse
addpath ../Framework/Waveforces
addpath ../Framework/MPC
addpath ../Framework/Plot
addpath ../Framework/Simulation
clc;
init_cybershipII




%% General
n_x = 6;
n_y = 3;
n_u = 3;
n_d = 2;
n_w = 9;
Ts = 0.8;
samples = 200;



Prob = struct;
Prob.SIMULATE = 'On';
Prob.name = 'PID using UKF estimate of current and waves';
Prob.disturbances.CURRENT = 'On';
Prob.disturbances.WAVES = 'On';
Prob.disturbances.WAVEDRIFT = 'Off';
Prob.ref.operation = 'SK'; % SK, Ellipsis, Path
Prob.simulation.eta_0 = [-5; 2; -pi/2];
Prob.simulation.nu_0 = zeros(n_u,1);
Prob.controller.TYPE = 'PID';
Prob.estimation.UKF_ONLINE = 'On';
Prob.estimation.UKF_OFFLINE = 'Off';
Prob.estimation.EKF_OFFLINE = 'Off';
Prob.controller.CONSTRAINED = 'Off';
Prob.estimation.PE_ONLINE = 'Off';
Prob.estimation.PE_OFFLINE = 'Off';

Prob.estimation.d_0 = 0*[0.5*cos(pi/3); 0.5*sin(pi/3)];

% Estimated values used in controller, 
Prob.estimation.S_ESTIMATE_ONLINE = 'On'; 
Prob.estimation.D_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC
Prob.estimation.B_ESTIMATE_ONLINE = 'Off'; % ONLY FOR MPC

%% Functions, GOOD!
Prob.simulation.integrator = @runge_kutta_4; Prob.simulation.ffun = @ffun2_s;

%% Variables, GOOD!
Prob.simulation.Ts = Ts;Prob.simulation.samples = samples;Prob.simulation.n_x = n_x;
Prob.simulation.n_y = n_y;Prob.simulation.n_u = n_u;
Prob.simulation.n_w = n_w;Prob.simulation.n_d = n_d;
Prob.simulation.t = 0:Ts:(samples-1)*Ts;


%% Reference generator, GOOD!
Prob.ref.ref_generator = @ref_generator;
switch Prob.ref.operation
    case 'SK' % SK
        Prob.ref.x_d = 0; Prob.ref.y_d = 0; Prob.ref.psi_d = 0;
        Prob.ref.Q = diag([5 5 5*(180/pi)^2])*10^3; % Inkludere farta?
        Prob.ref.R = 0*diag([1 1 0.01])*10^-6; Prob.ref.S = 0*diag([1 1 0.01])*10^-2;
    case 'Ellipsis'% Ellipsis
        Prob.ref.par_a = 50; Prob.ref.par_b = 50; Prob.ref.speed = 1;
        Prob.ref.Q = diag([5 5 5*(180/pi)^2]);
        Prob.ref.R = 0*diag([1 1 0.01])*10^-6; Prob.ref.S = 0*diag([1 1 0.01])*10^-2;
    case 'Path' % Path
        Prob.ref.speed = 1;
        Prob.ref.Q = diag([5 5 5*(180/pi)^2 1 1 1*(180/pi)^2])*10^3; % Inkludere farta?
        Prob.ref.R = diag([1 1 0.01])*10^-6; Prob.ref.S = diag([1 1 0.01])*10^-2;
end
Prob.ref.ref_dimension = size(Prob.ref.Q,1);
%% Disturbances, GOOD!
% General additive noise
Prob.disturbances.w_c_cov = zeros(n_x,1); % finn ekte verdiar
Prob.disturbances.v_c_cov = 0*[0.2^2 0.2^2 (0.025*pi/180)^2 0.10^2 0.10^2 (0.1*pi/180)^2]; % finn ekte verdiar

% Current
if strcmp(Prob.disturbances.CURRENT,'On')
    Prob.disturbances.dfun = @dfun_sim;
    Prob.disturbances.Tbvv = 500; Prob.disturbances.Tbbb = 500;
    Prob.disturbances.V_c_cov = 0.005; Prob.disturbances.beta_c_cov = pi/900;
    Prob.disturbances.V_c_0 = 0.5; Prob.disturbances.beta_c_0 = pi/3;
end

% Waves
if strcmp(Prob.disturbances.WAVES,'On')
    Prob.disturbances.waveforces = @waveForces; % Function
    Prob.disturbances.wavedir_0 = 3*pi/2; % Main wave-direction
    Prob.disturbances.shortcrest = 0; % Spreading on/off
    Prob.disturbances.DOF = n_y; % 3DOF or 6DOF waveforces
    Prob.disturbances.tFinal = 60*pi/2; % N_waves ~~ 2*tFinal/pi;
end
%% Estimation
%UKF, not GOOD, SDW
if strcmp(Prob.estimation.UKF_ONLINE,'On')
    Q_s = diag([0 0 0 1e-8 1e-8 1e-10]);
    Q_d = diag([0.000015^2 0.000015^2]);

    R_pose = diag([0.25^2 0.25^2 (0.2*pi/180)^2]);

    S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ukf = diag([0.5^2 0.5^2]);

    % Sette initialverdiar
    Prob.estimation.eta_0 = Prob.simulation.eta_0;
    Prob.estimation.nu_0 = zeros(3,1);
    
    %% UKF SD, modulus
    n_s = n_x + n_d;
    Prob.estimation.ukf_algorithm_online = @ukf_an;
    Prob.estimation.ukf_update_function_online = @ffun_sd;
    Prob.estimation.ukf_measurement_function_online = @hfun_pose;
    Prob.estimation.ukf_integrator_online = @runge_kutta_4;
    Prob.estimation.ukf_constraints_online = @constraints_velocity_current;
    Prob.estimation.ukf_ns_online = n_s; % Number of estimated states
    Prob.estimation.ukf_ny_online = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ukf_nx_online = n_x;
    Prob.estimation.ukf_nd_online = n_d;
    Prob.estimation.ukf_parameters_online = 'zeros(5,1)';
    
    if strcmp(Prob.estimation.S_ESTIMATE_ONLINE,'On')
        Prob.estimation.s_estimate = 'result.x_ukf_online(1:n_x,k)';
    end
    if strcmp(Prob.estimation.D_ESTIMATE_ONLINE,'On')
        Prob.estimation.d_estimate = 'result.x_ukf_online(n_x+1:n_x+n_d,k)';
    end
    if strcmp(Prob.estimation.B_ESTIMATE_ONLINE,'On')
        Prob.estimation.b_estimate = 'result.x_ukf_online(end-n_u+1:end,k)';
    end

    % Sette kovariansar
    Prob.estimation.ukf_Q_online = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ukf_R_online = R_pose; % Measurement noise covariance
    Prob.estimation.X_ukf_0_online = blkdiag(S_ukf,D_ukf);
end
%%
if strcmp(Prob.estimation.UKF_OFFLINE,'On')
%      Q_s = 0*ones(6)*1e-6;
%     Q_s = blkdiag(zeros(3),diag([0.01 0.01 0.0*(pi/180)^2]));
%       Q_s = diag([1e-5 1e-5 1e-7 0*1e-5 0*1e-5 0*1e-5*(pi/180)^2]);
% 
%     Q_d = diag([0.015^2 0.015^2]);
%     Q_w = diag([0 0 0 1 1 0.1 100 400 400]);
%     Q_w = diag([0 0 0 0.025^2 0.025^2 (0.024*pi/180)^2 100^2 350^2 500^2]);
%     R_pose = diag([5 5 (5*pi/180)^2]);
%     R_velocity = diag([0.10^2 0.10^2 (0.01*pi/180)^2]);
%     S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
%     D_ukf = diag([0.5^2 0.5^2]);
%     W_ukf = diag([1 1 1 0.5^2 0.5^2 (0.5*pi/180)^2 200^2 1000^2 1000^2]);

    
    Q_s = diag([0 0 0 0 0 0]);
    Q_d = diag([0.001^2 0.001^2]);
%     Q_w = diag([0 0 0 0.25^2 0.25^2 (0.2*pi/180)^2 100^2 100^2 200^2]);
    R_pose = diag([0.25^2 0.25^2 (0.15*pi/180)^2]);
%     R_velocity = diag([0.25^2 0.25^2 (0.5*pi/180)^2]);

    S_ukf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ukf = diag([0.5^2 0.5^2]);
%     W_ukf = diag([0.01 0.01 0.01 1 1 1*pi/180 1000^2 1000^2 2000^2]);

    
    n_s = n_x + n_d;
    Prob.estimation.ukf_algorithm_offline{1} = @ukf_an;
    Prob.estimation.ukf_update_function_offline{1} = @ffun_sd;
    Prob.estimation.ukf_measurement_function_offline{1} = @hfun_pose;
    Prob.estimation.ukf_integrator_offline{1} = @runge_kutta_4;
    Prob.estimation.ukf_constraints_offline{1} = @constraints_velocity_current;
    Prob.estimation.ukf_ns_offline{1} = n_s; % Number of estimated states
    Prob.estimation.ukf_ny_offline{1} = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ukf_nx_offline{1} = n_x;
    Prob.estimation.ukf_nd_offline{1} = n_d;
    Prob.estimation.ukf_parameters_offline{1} = 'zeros(5,1)';

    % Sette kovariansar
    Prob.estimation.ukf_Q_offline{1} = blkdiag(Q_s,Q_d); % Process noise covariance
    Prob.estimation.ukf_R_offline{1} = blkdiag(R_pose); % Measurement noise covariance
    Prob.estimation.X_ukf_0_offline{1} = blkdiag(S_ukf,D_ukf);
end
%% EKF, not GOOD, SDW
if strcmp(Prob.estimation.EKF_OFFLINE,'On')
    Q_s_ekf = blkdiag(diag([0.2 0.2 0.2]),diag([3 3 (180/pi)^2])*10^-1);
    Q_d_ekf = diag([0.0015^2 0.0015^2]);
    Q_w_ekf = diag([0 0 0 1 1 1*(pi/180)^2 100 200 400]);
    R_pose_ekf = diag([0.5^2 0.5^2 (0.5*pi/180)^2]);
    R_velocity_ekf = diag([0.10^2 0.10^2 (0.1*pi/180)^2]);

    S_ekf = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); % Belief in initial guess, low value = high belief
    D_ekf = diag([2 2]);
    W_ekf = diag([0.01 0.01 0.01 1 1 1*pi/180 2000^2 8000^2 8000^2]);

    % Sette initialverdiar
    Prob.estimation.eta_0 = Prob.simulation.eta_0;
    Prob.estimation.nu_0 = zeros(3,1);

    %% EKF SDW, modulus
    n_s = n_x + n_d + n_w;
    Prob.estimation.ekf_algorithm_offline = @ekf;
    Prob.estimation.ekf_update_function_offline = @ffun_sdw_ekf;
    Prob.estimation.ekf_measurement_function_offline = @hfun_pose_waves;
    Prob.estimation.ekf_update_jacobian_offline = @ffun_sdw_ekf_jacobian;
    Prob.estimation.ekf_measurement_jacobian_offline = @hfun_pose_waves_jacobian;
    Prob.estimation.ekf_integrator_offline = @runge_kutta_4;
    Prob.estimation.ekf_constraints_offline = @constraints_velocity_current_waves;
    Prob.estimation.ekf_ns_offline = n_s; % Number of estimated states
    Prob.estimation.ekf_ny_offline = n_y; % Number of measurements 3 = n_y/6 = n_x
    Prob.estimation.ekf_nx_offline = n_x;
    Prob.estimation.ekf_nd_offline = n_d;
    Prob.estimation.ekf_parameters_offline = 'zeros(5,1)';

    % Sette kovariansar
    Prob.estimation.ekf_Q_offline = blkdiag(Q_s,Q_d,Q_w); % Process noise covariance
    Prob.estimation.ekf_R_offline = R_pose; % Measurement noise covariance
    Prob.estimation.X_ekf_0_offline = blkdiag(S_ukf,D_ukf,W_ukf);
end

%% PID kontroller
if strcmp(Prob.controller.TYPE,'PID')
    Prob.controller.n_p = 1;
    wb_x = 0.25; wb_y = 0.20; wb_psi = 15*pi/180;
    eta_x = 0.790; eta_y = 0.790; eta_psi = 0.790;
    wn_x = wb_x/sqrt(1-2*eta_x^2 + sqrt(4*eta_x^4 - 4*eta_x^2 + 2));
    wn_y = wb_y/sqrt(1-2*eta_y^2 + sqrt(4*eta_y^4 - 4*eta_y^2 + 2));
    wn_psi = wb_psi/sqrt(1-2*eta_psi^2 + sqrt(4*eta_psi^4 - 4*eta_psi^2 + 2));
    m_x = cs.m+cs.X_udot; m_y = cs.m+cs.Y_vdot; m_psi = cs.I_z+cs.N_rdot;
    d_x = cs.X3_u; d_y = cs.Y3_v; d_psi = cs.N3_r;
    Km_x = 0; Km_y = 0; Km_psi = 0;
    Kp_x = (m_x + Km_x)*wn_x^2; Kp_y = (m_y + Km_y)*wn_y^2;  Kp_psi = (m_psi + Km_psi)*wn_psi^2;
    Kd_x = ((m_x + Km_x)*wn_x*2*eta_x - d_x); Kd_y = ((m_y + Km_y)*wn_y*2*eta_y - d_y);  Kd_psi = ((m_psi + Km_psi)*wn_psi*2*eta_psi - d_psi);
    Ki_x = wn_x/10*Kp_x; Ki_y = wn_y/10*Kp_y; Ki_psi = wn_psi/10*Kp_psi;

    Prob.controller.Kp = diag([Kp_x; Kp_y; Kp_psi]);
    Prob.controller.Kd = diag([Kd_x; Kd_y; Kd_psi]);
    Prob.controller.Ki = diag([Ki_x; Ki_y/2; Ki_psi/2]);
end
%%

if strcmp(Prob.SIMULATE,'On')
    result = simulate(Prob);
    save sim_result_waves_good result
else
    load sim_result_waves_good
end
%%

for k = 1:samples;
    if strcmp(Prob.estimation.UKF_OFFLINE,'On');  
        UKF_estimation_offline; 
%         result.x_ukf_offline{1}(9:14,k) 
    end

    if strcmp(Prob.estimation.EKF_OFFLINE,'On'); EKF_estimation_offline; end

    if strcmp(Prob.estimation.PE_OFFLINE,'On'); PE_estimation_offline; end
end
%%
plot_disturbances(result.t,result.d_sim_ned,'k-',1);
plot_disturbances(result.t,result.x_ukf_online(7:8,:),'b:',1);

plot_states_controller(result.t,result.ref_sim,'k-',3);
plot_states_controller(result.t,result.x_sim,'m-.',3);

plot_states_est(result.t,result.x_sim,'k-',4);
plot_states_est(result.t,result.x_ukf_online(1:6,:),'b:',4);
